# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-13 12:25
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ContactMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('author', models.CharField(max_length=256)),
                ('email', models.EmailField(max_length=256)),
                ('subject', models.CharField(max_length=256)),
                ('status', models.CharField(choices=[('STATUS_ACTIVE', 'active'), ('STATUS_CLOSED', 'closed')], max_length=8)),
                ('created', models.DateTimeField(blank=True, null=True)),
                ('message', models.TextField()),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='contact_messages', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('created', 'status'),
                'verbose_name': 'Contact message',
                'verbose_name_plural': 'Contact messages',
            },
        ),
    ]



from django.utils.translation import gettext_lazy as _

from daiquiri.core.utils import make_query_dict_upper_case


class BaseServiceAdapter:

    def clean(self, request, resource):
        raise NotImplementedError()

    def stream(self):
        raise NotImplementedError()

    def parse_query_dict(self, request):
        data = make_query_dict_upper_case(request.GET)

        args = {}
        errors = {}
        for key in self.keys:
            try:
                value = float(data[key])

                if self.ranges[key]['min'] <= value <= self.ranges[key]['max']:
                    args[key] = value
                else:
                    errors[key] = [_('This value must be between {min:g} and {max:g}.').format(**self.ranges[key])]

            except KeyError:
                if key in self.defaults:
                    args[key] = self.defaults[key]
                else:
                    errors[key] = [_('This field may not be blank.')]

            except ValueError:
                errors[key] = [_('This field must be a float.')]

        return args, errors



# Generated by Django 2.2.8 on 2021-12-02 16:10

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0027_auto_20201202_1625'),
    ]

    operations = [
        migrations.AddField(
            model_name='schema',
            name='related_identifiers',
            field=jsonfield.fields.JSONField(blank=True, null=True, verbose_name='Related Identifiers'),
        ),
        migrations.AddField(
            model_name='table',
            name='related_identifiers',
            field=jsonfield.fields.JSONField(blank=True, null=True, verbose_name='Related Identifiers'),
        ),
        migrations.AlterField(
            model_name='schema',
            name='license',
            field=models.CharField(blank=True, choices=[('CC0', 'CC0 1.0 Universal (CC0 1.0)'), ('PD', 'Public Domain Mark'), ('BY', 'Attribution 4.0 International (CC BY 4.0)'), ('BY_SA', 'Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)'), ('BY_ND', 'Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)'), ('BY_NC', 'Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)'), ('BY_NC_SA', 'Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)'), ('BY_NC_ND', 'Attribution-NonCommercial-NoDerivatives 4.0 International (CC BY-NC-ND 4.0)')], max_length=8, null=True, verbose_name='License'),
        ),
        migrations.AlterField(
            model_name='table',
            name='license',
            field=models.CharField(blank=True, choices=[('CC0', 'CC0 1.0 Universal (CC0 1.0)'), ('PD', 'Public Domain Mark'), ('BY', 'Attribution 4.0 International (CC BY 4.0)'), ('BY_SA', 'Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)'), ('BY_ND', 'Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)'), ('BY_NC', 'Attribution-NonCommercial 4.0 International (CC BY-NC 4.0)'), ('BY_NC_SA', 'Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0)'), ('BY_NC_ND', 'Attribution-NonCommercial-NoDerivatives 4.0 International (CC BY-NC-ND 4.0)')], max_length=8, null=True, verbose_name='License'),
        ),
        migrations.AlterField(
            model_name='table',
            name='type',
            field=models.CharField(choices=[('table', 'table'), ('view', 'view')], max_length=8, verbose_name='Type of table'),
        ),
    ]



import pytest

from django.urls import reverse

from ..models import Profile

users = (
    ('admin', 'admin'),
    ('manager', 'manager'),
    ('user', 'user'),
    ('anonymous', None),
)

status_map = {
    'list': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'detail': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'create': {
        'admin': 405, 'manager': 403, 'user': 403, 'anonymous': 403
    },
    'update': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'delete': {
        'admin': 405, 'manager': 403, 'user': 403, 'anonymous': 403
    },
    'confirm': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'reject': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'activate': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'disable': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    },
    'enable': {
        'admin': 200, 'manager': 200, 'user': 403, 'anonymous': 403
    }
}

urlnames = {
    'list': 'auth:profile-list',
    'detail': 'auth:profile-detail',
    'confirm': 'auth:profile-confirm',
    'reject': 'auth:profile-reject',
    'activate': 'auth:profile-activate',
    'enable': 'auth:profile-enable',
    'disable': 'auth:profile-disable'
}

instances = [5]


@pytest.mark.parametrize(('username', 'password'), users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)
    assert response.status_code == status_map['list'][username], response.json()

    if response.status_code == 200:
        assert len(response.json()) == 4


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_detail(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Profile.objects.get(pk=pk)

    url = reverse(urlnames['detail'], args=[pk])
    response = client.get(url)
    assert response.status_code == status_map['detail'][username], response.json()

    if response.status_code == 200:
        assert response.json().get('user').get('username') == instance.user.username


@pytest.mark.parametrize(('username', 'password'), users)
def test_create(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url, {})
    assert response.status_code == status_map['create'][username], response.json()


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_update(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Profile.objects.get(pk=pk)
    instance_first_name = instance.user.first_name

    url = reverse(urlnames['detail'], args=[pk])
    response = client.put(url, {
        'attributes': instance.attributes,
        'details': instance.details,
        'user': {
            'first_name': 'Test',
            'last_name': instance.user.last_name,
            'groups': [group.id for group in instance.user.groups.all()],
        }
    }, content_type='application/json')
    assert response.status_code == status_map['update'][username], response.json()

    instance.refresh_from_db()
    assert instance.user.first_name == ('Test' if response.status_code == 200 else instance_first_name)


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_delete(db, client, username, password, pk):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[pk])
    response = client.delete(url)
    assert response.status_code == status_map['delete'][username], response.json()
    assert Profile.objects.filter(pk=pk).exists() is True


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_confirm(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Profile.objects.get(pk=pk)
    instance.is_pending = True
    instance.is_confirmed = False
    instance.user.save()
    instance.save()

    url = reverse(urlnames['confirm'], args=[pk])
    response = client.put(url)
    assert response.status_code == status_map['confirm'][username], response.json()

    instance.refresh_from_db()
    assert instance.is_pending is True
    assert instance.is_confirmed == (response.status_code == 200)
    assert instance.user.is_active is True


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_reject(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Profile.objects.get(pk=pk)
    instance.is_pending = True
    instance.is_confirmed = False
    instance.user.is_active = True
    instance.user.save()
    instance.save()

    url = reverse(urlnames['reject'], args=[pk])
    response = client.put(url)
    assert response.status_code == status_map['reject'][username], response.json()

    instance.refresh_from_db()
    assert instance.is_pending is not (response.status_code == 200)
    assert instance.is_confirmed is False
    assert instance.user.is_active is not (response.status_code == 200)


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_activate(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Profile.objects.get(pk=pk)
    instance.is_pending = True
    instance.is_confirmed = False
    instance.user.save()
    instance.save()

    url = reverse(urlnames['activate'], args=[pk])
    response = client.put(url)
    assert response.status_code == status_map['activate'][username], response.json()

    instance.refresh_from_db()
    assert instance.is_pending is not (response.status_code == 200)
    assert instance.is_confirmed is (response.status_code == 200)
    assert instance.user.is_active is True


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_enable(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Profile.objects.get(pk=pk)
    instance.user.is_active = False
    instance.user.save()

    url = reverse(urlnames['enable'], args=[pk])
    response = client.put(url)
    assert response.status_code == status_map['enable'][username], response.json()

    instance.refresh_from_db()
    assert instance.is_pending is False
    assert instance.is_confirmed is False
    assert instance.user.is_active is (response.status_code == 200)


@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('pk', instances)
def test_disable(db, client, username, password, pk):
    client.login(username=username, password=password)

    instance = Profile.objects.get(pk=pk)

    url = reverse(urlnames['disable'], args=[pk])
    response = client.put(url)
    assert response.status_code == status_map['disable'][username], response.json()

    instance.refresh_from_db()
    assert instance.is_pending is False
    assert instance.is_confirmed is False
    assert instance.user.is_active is not (response.status_code == 200)



import daiquiri.core.env as env

FILES_BASE_PATH = env.get_abspath('FILES_BASE_PATH')
FILES_BASE_URL = env.get('FILES_BASE_URL')

FILES_DOCS_REL_PATH = env.get('FILES_DOCS_REL_PATH')
FILES_SEARCH_RESULTS_PER_PAGE = 5



from django.utils.deprecation import MiddlewareMixin


class MultipleProxyMiddleware(MiddlewareMixin):
    """
    see also: https://docs.djangoproject.com/en/2.1/ref/request-response/
    (but using the left most entry)
    """

    FORWARDED_FOR_FIELDS = [
        'HTTP_X_FORWARDED_FOR',
        'HTTP_X_FORWARDED_HOST',
        'HTTP_X_FORWARDED_SERVER',
    ]

    def process_request(self, request):
        """
        Rewrites the proxy headers so that only the most recent proxy is used.
        """
        for field in self.FORWARDED_FOR_FIELDS:
            if field in request.META:
                request.META[field] = request.META[field].split(',')[0].strip()



import os
import subprocess

from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('command', nargs="*")

    def handle(self, *args, **options):
        nvm_dir = os.getenv('NVM_DIR')

        if nvm_dir is None:
            raise CommandError('NVM_DIR is not set, is nvm.sh installed?')

        if not os.path.exists(nvm_dir):
            raise CommandError('NVM_DIR does not exist, is nvm.sh installed?')

        command = ' '.join(options['command'])
        subprocess.call(['/bin/bash', '-c', f'source {nvm_dir}/nvm.sh; npm {command}'])



import uuid

from django.contrib.auth.models import User
from django.db import models
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _


class Job(models.Model):

    PHASE_PENDING = 'PENDING'
    PHASE_QUEUED = 'QUEUED'
    PHASE_EXECUTING = 'EXECUTING'
    PHASE_COMPLETED = 'COMPLETED'
    PHASE_ERROR = 'ERROR'
    PHASE_ABORTED = 'ABORTED'
    PHASE_UNKNOWN = 'UNKNOWN'
    PHASE_HELD = 'HELD'
    PHASE_SUSPENDED = 'SUSPENDED'
    PHASE_ARCHIVED = 'ARCHIVED'
    PHASE_RUN = 'RUN'
    PHASE_ABORT = 'ABORT'
    PHASE_ACTIVE = (
        PHASE_QUEUED,
        PHASE_EXECUTING
    )
    PHASE_CHOICES = (
        (PHASE_PENDING, _('Pending')),
        (PHASE_QUEUED, _('Queued')),
        (PHASE_EXECUTING, _('Executing')),
        (PHASE_COMPLETED, _('Completed')),
        (PHASE_ERROR, _('Error')),
        (PHASE_ABORTED, _('Aborted')),
        (PHASE_UNKNOWN, _('Unknown')),
        (PHASE_HELD, _('Held')),
        (PHASE_SUSPENDED, _('Suspended')),
        (PHASE_ARCHIVED, _('Archived'))
    )

    JOB_TYPE_SYNC = 'SYNC'
    JOB_TYPE_ASYNC = 'ASYNC'
    JOB_TYPE_INTERFACE = 'INTERFACE'
    JOB_TYPE_CHOICES = (
        (JOB_TYPE_SYNC, _('Synchronous')),
        (JOB_TYPE_ASYNC, _('Asynchronous')),
        (JOB_TYPE_INTERFACE, _('Interface')),
    )

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    owner = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE)

    client_ip = models.GenericIPAddressField(blank=True, null=True)

    response_format = models.CharField(max_length=64, blank=True, null=True)  # noqa: DJ001
    max_records = models.IntegerField(blank=True, null=True)
    run_id = models.CharField(max_length=64, blank=True, default='', db_index=True)

    phase = models.CharField(max_length=10, choices=PHASE_CHOICES, db_index=True)

    creation_time = models.DateTimeField(blank=True, null=True, db_index=True)
    start_time = models.DateTimeField(blank=True, null=True)
    end_time = models.DateTimeField(blank=True, null=True)
    execution_duration = models.PositiveIntegerField(blank=True, default=0)
    destruction_time = models.DateTimeField(blank=True, null=True)

    error_summary = models.TextField(blank=True, null=True)  # noqa: DJ001

    job_type = models.CharField(max_length=10, choices=JOB_TYPE_CHOICES)

    class Meta:
        ordering = ('start_time', )

        verbose_name = _('Job')
        verbose_name_plural = _('Jobs')

    def __str__(self):
        return str(self.id)

    def save(self, *args, **kwargs):
        if self.pk is None:
            self.phase = self.PHASE_PENDING
            self.creation_time = now()

        return super().save(*args, **kwargs)

    @property
    def owner_username(self):
        return self.owner.username if self.owner else 'anonymous'

    @property
    def parameters(self):
        raise NotImplementedError

    @property
    def formats(self):
        raise NotImplementedError

    @property
    def quote(self):
        raise NotImplementedError

    def process(self):
        raise NotImplementedError

    def run(self):
        raise NotImplementedError

    def run_sync(self):
        raise NotImplementedError

    def abort(self):
        raise NotImplementedError

    def archive(self):
        raise NotImplementedError

    def stream(self):
        raise NotImplementedError



from django.conf import settings

from rest_framework.permissions import BasePermission


class HasPermission(BasePermission):

    def has_permission(self, request, view):
        return settings.QUERY_ANONYMOUS or (request.user and request.user.is_authenticated)



from urllib.parse import urlparse

from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    requires_system_checks = []
    can_import_settings = True

    def handle(self, *args, **options):

        parsed_url = urlparse(settings.CELERY_BROKER_URL)

        vhost = parsed_url.path.lstrip('/')

        print(f'rabbitmqctl add_user {parsed_url.username} {parsed_url.password}')
        print(f'rabbitmqctl add_vhost {vhost}')
        print(f'rabbitmqctl set_permissions -p {vhost} {parsed_url.username} ".*" ".*" ".*"')
        print(f'rabbitmqctl set_permissions -p {vhost} admin ".*" ".*" ".*"')



from django.urls import include, path, re_path

from rest_framework import routers

from .views import QueryView
from .viewsets import (
    DownloadViewSet,
    DropdownViewSet,
    ExampleViewSet,
    FormViewSet,
    PhaseViewSet,
    QueryDownloadFormatViewSet,
    QueryJobViewSet,
    QueryLanguageViewSet,
    QueueViewSet,
    StatusViewSet,
)

app_name = 'query'

router = routers.DefaultRouter()
router.register(r'status', StatusViewSet, basename='status')
router.register(r'forms', FormViewSet, basename='form')
router.register(r'dropdowns', DropdownViewSet, basename='dropdown')
router.register(r'downloads', DownloadViewSet, basename='download')
router.register(r'downloadformats', QueryDownloadFormatViewSet, basename='downloadformat')
router.register(r'jobs', QueryJobViewSet, basename='job')
router.register(r'examples', ExampleViewSet, basename='example')
router.register(r'queues', QueueViewSet, basename='queue')
router.register(r'querylanguages', QueryLanguageViewSet, basename='querylanguage')
router.register(r'phases', PhaseViewSet, basename='phase')

urlpatterns = [
    # rest api
    path(r'api/', include(router.urls)),

    # query interface, needs to be last in list
    re_path(r'[A-Za-z0-9-]*/$', QueryView.as_view(), name='query'),
]



DATALINK_CONTENT_TYPE = 'application/x-votable+xml;content=datalink'
DATALINK_TABLE = {
    'name': 'datalink',
    'description': 'The table of datalinks for the service',
    'order': 4,
    'access_level': 'PUBLIC',
    'metadata_access_level': 'PUBLIC'
}
DATALINK_FIELDS = [
    {
        'name': 'ID',
        'order': 1,
        'description': 'Input identifier.',
        'ucd': 'meta.id;meta.main',
        'datatype': 'char',
        'arraysize': None,
        'std': True,
        'access_level': 'PUBLIC',
        'metadata_access_level': 'PUBLIC'
    },
    {
        'name': 'access_url',
        'order': 2,
        'description': 'Link to data or service.',
        'ucd': 'meta.ref.url',
        'datatype': 'char',
        'arraysize': None,
        'std': True,
        'access_level': 'PUBLIC',
        'metadata_access_level': 'PUBLIC'
    },
    {
        'name': 'service_def',
        'order': 3,
        'description': 'Reference to a service descriptor resource.',
        'ucd': 'meta.ref',
        'datatype': 'char',
        'arraysize': None,
        'std': True,
        'access_level': 'PUBLIC',
        'metadata_access_level': 'PUBLIC'
    },
    {
        'name': 'error_message',
        'order': 4,
        'description': 'Error if an access_url cannot be created.',
        'ucd': 'meta.code.error',
        'datatype': 'char',
        'arraysize': None,
        'std': True,
        'access_level': 'PUBLIC',
        'metadata_access_level': 'PUBLIC'
    },
    {
        'name': 'description',
        'order': 5,
        'description': 'Human-readable text describing this link.',
        'ucd': 'meta.note',
        'datatype': 'char',
        'arraysize': None,
        'std': True,
        'access_level': 'PUBLIC',
        'metadata_access_level': 'PUBLIC'
    },
    {
        'name': 'semantics',
        'order': 6,
        'description': 'Term from a controlled vocabulary describing the link.',
        'ucd': 'meta.code',
        'datatype': 'char',
        'arraysize': None,
        'std': True,
        'access_level': 'PUBLIC',
        'metadata_access_level': 'PUBLIC'
    },
    {
        'name': 'content_type',
        'order': 7,
        'description': 'Mime-type of the content the link returns.',
        'ucd': 'meta.code.mime',
        'datatype': 'char',
        'arraysize': None,
        'std': True,
        'access_level': 'PUBLIC',
        'metadata_access_level': 'PUBLIC'
    },
    {
        'name': 'content_length',
        'order': 8,
        'description': 'Size of the download the link returns.',
        'ucd': 'phys.size;meta.file',
        'datatype': 'long',
        'arraysize': None,
        'unit': 'byte',
        'std': True,
        'access_level': 'PUBLIC',
        'metadata_access_level': 'PUBLIC'
    }
]

DATALINK_RELATION_TYPES = {
    'IsCitedBy': '{} is cited by the linked resource.',
    'Cites': '{} cites the linked resource.',
    'IsSupplementTo': '{} is supplement to the linked resource.',
    'IsSupplementedBy': '{} is supplemented by the linked resource.',
    'IsContinuedBy': '{} is continued by the linked resource.',
    'Continues': '{} continues the linked resource.',
    'IsDescribedBy': '{} is described by the linked resource.',
    'Describes': '{} describes the linked resource.',
    'HasMetadata': 'the linked resource is metadata for {}.',
    'IsMetadataFor': '{} is metadata for the linked resource.',
    'HasVersion': 'the linked resource is a version of {}.',
    'IsVersionOf': '{} is a version of the linked resource.',
    'IsNewVersionOf': '{} is a newer version of the linked resource.',
    'IsPreviousVersionOf': '{} is a previous version of the linked resource.',
    'IsPartOf': '{} is part of the linked resource.',
    'HasPart': 'the linked resource is a part of {}.',
    'IsPublishedIn': '{} is published in the linked resource.',
    'IsReferencedBy': '{} is referenced by the linked resource.',
    'References': '{} references the linked resource.',
    'IsDocumentedBy': '{} is documented by the linked resource.',
    'Documents': '{} documents the linked resource.',
    'IsCompiledBy': '{} is compiled by the linked resource.',
    'Compiles': '{} compiles the linked resource.',
    'IsVariantFormOf': '{} is a variant form of the linked resource.',
    'IsOriginalFormOf': '{} is the original form of the linked resource.',
    'IsIdenticalTo': '{} is identical to the linked resource.',
    'IsReviewedBy': '{} is reviewed by the linked resource.',
    'Reviews': '{} reviews the linked resource.',
    'IsDerivedFrom': '{} is derived from the linked resource.',
    'IsSourceOf': '{} is the source of the linked resource.',
    'IsRequiredBy': '{} is required by the linked resource.',
    'Requires': '{} requires the linked resource.',
    'IsObsoletedBy': '{} is obsoleted by the linked resource.',
    'Obsoletes': '{} obsoletes the linked resource.'
}



from datetime import datetime
from urllib.parse import quote, unquote

from django.apps import apps
from django.conf import settings
from django.http.request import QueryDict
from django.utils.http import urlencode

from rest_framework.response import Response
from rest_framework.views import APIView

from .adapter import OaiAdapter
from .models import Record
from .renderers import OaiRenderer
from .utils import get_metadata_format, get_renderer


class OaiView(APIView):

    def get(self, request):
        return self.get_response(request, request.GET)

    def post(self, request):
        return self.get_response(request, request.POST)

    def get_response(self, request, query_dict):
        # initialize response and errors array
        self.response = None
        self.errors = []

        verb, arguments = self.get_verb_and_arguments(query_dict)

        if verb is None:
            self.errors.append(('badArgument', 'OAI verb missing'))
        elif verb == 'GetRecord':
            self.get_record(arguments)
        elif verb == 'Identify':
            self.identify(arguments)
        elif verb == 'ListIdentifiers':
            self.list_records(arguments, metadata=False)
        elif verb == 'ListMetadataFormats':
            self.list_metadata_formats(arguments)
        elif verb == 'ListRecords':
            self.list_records(arguments)
        elif verb == 'ListSets':
            self.list_sets(arguments)
        else:
            self.errors.append(('badVerb', 'Illegal OAI verb'))

        if self.response is not None and 'metadataPrefix' in self.response:
            renderer = get_renderer(self.response['metadataPrefix'])
        else:
            renderer = OaiRenderer()

        request.accepted_renderer = renderer
        request.accepted_media_type = renderer.media_type

        return Response({
            'responseDate': datetime.utcnow().replace(microsecond=0).isoformat() + 'Z',
            'baseUrl': request.build_absolute_uri(request.path),
            'verb': verb,
            'arguments': arguments,
            'errors': self.errors,
            'response': self.response
        })

    def get_verb_and_arguments(self, query_dict):
        # validate keys in params
        arguments = {}
        for key in query_dict:
            if len(query_dict.getlist(key)) > 1:
                if key == 'verb':
                    self.errors.append(('badVerb', 'Found illegal duplicate of verb'))
                else:
                    self.errors.append(('badArgument', f'Found illegal duplicate of argument \'{key}\'.'))
            else:
                arguments[key] = query_dict.get(key)

        verb = arguments.pop('verb', None)

        return verb, arguments

    def identify(self, arguments):
        self.validate_illegal_arguments(arguments, [])

        if self.errors:
            return

        adapter = OaiAdapter()

        earliest_record = Record.objects.order_by('datestamp').first()

        self.response = {
            'repository_name': settings.SITE_IDENTIFIER,
            'admin_email': settings.SITE_CONTACT['email'],
            'earliest_datestamp': earliest_record.datestamp if earliest_record else None,
            'deleted_record': settings.OAI_DELETED_RECORD,
            'identifier': {
                'scheme': adapter.identifier_schema,
                'repository_identifier': adapter.identifier_repository,
                'delimiter': adapter.identifier_delimiter,
                'sample_identifier': adapter.get_sample_identifier()
            },
            'registry': self.get_registry()
        }

    def get_record(self, arguments):
        self.validate_illegal_arguments(arguments, ['identifier', 'metadataPrefix'])
        self.validate_identifier(arguments)
        self.validate_metadata_prefix(arguments)

        if self.errors:
            return

        records = Record.objects.filter(identifier=arguments['identifier'])

        if not records.exists():
            self.errors.append(('idDoesNotExist', 'No item found with this identifier'))
            return

        try:
            record = records.get(metadata_prefix=arguments['metadataPrefix'])
        except Record.DoesNotExist:
            self.errors.append(
                ('cannotDisseminateFormat', 'No item found for this identifier with this metadataPrefix')
            )
            return

        self.response = {
            'header': self.get_header(record),
            'metadata': self.get_metadata(record),
            'metadataPrefix': arguments['metadataPrefix']
        }

    def list_metadata_formats(self, arguments):
        self.validate_illegal_arguments(arguments, ['identifier'])

        if self.errors:
            return

        if 'identifier' in arguments:
            records = Record.objects.filter(identifier=arguments['identifier'])

            if not records.exists():
                self.errors.append(('idDoesNotExist', 'No item found with this identifier'))
                return

            else:
                self.response = [get_metadata_format(record.metadata_prefix) for record in records]

        else:
            self.response = settings.OAI_METADATA_FORMATS

    def list_records(self, arguments, metadata=True):
        self.validate_illegal_arguments(arguments, ['from', 'until', 'metadataPrefix', 'set', 'resumptionToken'])
        arguments = self.validate_resumption_token(arguments)

        if self.errors:
            return

        from_date = self.validate_date('from', arguments)
        until_date = self.validate_date('until', arguments)
        set_spec = self.validate_set(arguments)
        self.validate_metadata_prefix(arguments)

        if self.errors:
            return

        records = Record.objects.filter(metadata_prefix=arguments['metadataPrefix'])

        if set_spec:
            records = records.filter(set_spec=set_spec)

        if from_date:
            records = records.filter(datestamp__gte=from_date)

        if until_date:
            records = records.filter(datestamp__lte=until_date)

        count = records.count()
        if count >= settings.OAI_PAGE_SIZE:
            start = int(arguments.pop('start', '0'))
            stop = start + settings.OAI_PAGE_SIZE

            records = records[start:stop]

            if stop < count:
                resumption_token = {
                    'completeListSize': count,
                    'cursor': start,
                    'token': quote(urlencode(dict(start=stop, **arguments)))
                }
            else:
                resumption_token = False

        else:
            resumption_token = False

        if not records.exists():
            self.errors.append(('noRecordsMatch', 'No items found'))
            return

        self.response = {
            'items': [
                {
                    'header': self.get_header(record),
                    'metadata': self.get_metadata(record) if metadata else None
                } for record in records
            ],
            'resumptionToken': resumption_token,
            'metadataPrefix': arguments['metadataPrefix']
        }

    def list_sets(self, arguments):
        self.validate_illegal_arguments(arguments, ['resumptionToken'])
        if settings.OAI_SETS:
            # need to be ordered, see note in
            # https://docs.djangoproject.com/en/3.0/ref/models/querysets/#django.db.models.query.QuerySet.distinct
            oai_sets = Record.objects.order_by('set_spec').values('set_spec').distinct('set_spec')
            self.response = {
                'oai_sets': [
                    {
                        'setSpec': oai_set['set_spec'],
                        'setName': oai_set['set_spec'],
                        'setDescription': None
                    } for oai_set in oai_sets
                ]
            }
        else:
            self.errors.append(('noSetHierarchy', 'This repository does not support sets'))

    def validate_illegal_arguments(self, arguments, valid_keys):
        for key in arguments:
            if key not in valid_keys:
                self.errors.append(('badArgument', f'Found illegal argument \'{key}\'.'))
        else:
            return None

    def validate_date(self, key, arguments):
        if key in arguments:
            try:
                return datetime.strptime(arguments[key], '%Y-%m-%dT%H:%M:%SZ').date()
            except ValueError:
                self.errors.append(('badArgument', f'Argument \'{key}\' does not match format YYYY-MM-DDThh:mm:ssZ'))
        else:
            return None

    def validate_resumption_token(self, arguments):
        if 'resumptionToken' in arguments:
            resumption_token = arguments.pop('resumptionToken')
            if arguments:
                self.errors.append(('badArgument', 'resumptionToken is an exclusive argument.'))

            query_dict = QueryDict(query_string=unquote(resumption_token), mutable=False)
            verb, arguments = self.get_verb_and_arguments(query_dict)
            if 'start' not in arguments:
                self.errors.append(('badResumptionToken', 'The value of the resumptionToken argument is invalid.'))

        return arguments

    def validate_set(self, arguments):
        if 'set' in arguments:
            if not settings.OAI_SETS:
                self.errors.append(('noSetHierarchy', 'This repository does not support sets'))
            else:
                return arguments['set']
        else:
            return None

    def validate_identifier(self, arguments):
        if 'identifier' not in arguments:
            self.errors.append(('badArgument', 'Argument \'identifier\' required but not supplied.'))

    def validate_metadata_prefix(self, arguments):
        valid_prefixes = [metadata_format['prefix'] for metadata_format in settings.OAI_METADATA_FORMATS]

        if 'metadataPrefix' not in arguments:
            self.errors.append(('badArgument', 'Argument \'metadataPrefix\' required but not supplied.'))
        elif arguments['metadataPrefix'] not in valid_prefixes:
            self.errors.append((
                'cannotDisseminateFormat',
                'The metadataPrefix \'{}\' is not supported by this repository.'.format(arguments['metadataPrefix'])
            ))

    def get_header(self, record):
        return {
            'identifier': record.identifier,
            'datestamp': record.datestamp,
            'deleted': record.deleted,
            'setSpec': [record.set_spec]
        }

    def get_metadata(self, record):
        if record.deleted:
            return None
        else:
            adapter = OaiAdapter()
            resource = adapter.get_resource(record)
            serializer_class = adapter.get_serializer_class(record.metadata_prefix, record.resource_type)
            if serializer_class is not None:
                serializer = serializer_class(instance=resource, context={
                    'request': self.request
                })
            else:
                raise RuntimeError(f'Could not determine serializer_class for record {record.identifier}')
            return serializer.data

    def get_registry(self):
        if apps.is_installed('daiquiri.registry'):
            from daiquiri.registry.vo import get_resource
            return get_resource()



import logging
import os
import zipfile

from celery import shared_task

from daiquiri.core.tasks import Task


@shared_task(track_started=True, base=Task)
def create_download_archive(file_name, files):
    # get logger
    logger = logging.getLogger(__name__)

    # log start
    logger.info('archive_job %s started', file_name)

    # create directory if necessary
    try:
        os.mkdir(os.path.dirname(file_name))
    except OSError:
        pass

    # create a zipfile with all files
    with zipfile.ZipFile(file_name, 'w') as z:
        for directory_path, file_path in files:
            os.chdir(directory_path)
            z.write(file_path)

    # log completion
    logger.info('archive_job %s completed', file_name)



# Generated by Django 4.0.8 on 2022-11-14 11:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_datalink', '0002_alter_datalink_content_length'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datalink',
            name='content_length',
            field=models.BigIntegerField(null=True, verbose_name='Content length'),
        ),
    ]



import re

from django.core.validators import URLValidator
from django.utils.translation import gettext_lazy as _

from rest_framework.exceptions import ValidationError

from .models import QueryJob
from .utils import get_quota


class TableNameValidator:

    requires_context = True

    message = _('A job with this table name already exists.')
    message_allowed_chars = _(
        'Please only use letters, numbers, hyphens or underscores.'
    )

    def __call__(self, table_name, serializer_field):
        request = serializer_field.parent.context['request']
        user = None if request.user.is_anonymous else request.user

        if serializer_field.parent.instance:
            current_table_name = serializer_field.parent.instance.table_name
        else:
            current_table_name = None

        if bool(re.search(r'^[0-9a-zA-Z_\-]+$', table_name)) is False:
            raise ValidationError([self.message_allowed_chars])
        if table_name:
            if current_table_name and table_name == current_table_name:
                pass
            else:
                try:
                    QueryJob.objects.filter(owner=user).exclude(phase=QueryJob.PHASE_ARCHIVED).get(table_name=table_name)
                    raise ValidationError([self.message])
                except QueryJob.DoesNotExist:
                    pass


class UploadFileValidator:

    requires_context = True

    message = _("The maximum file size that can be uploaded is %s.")

    def __call__(self, file, serializer_field):
        request = serializer_field.parent.context['request']
        user = None if request.user.is_anonymous else request.user

        quota = get_quota(user, quota_settings='QUERY_UPLOAD_LIMIT')
        if file.size > quota:
            raise ValidationError({
                'file': self.message % quota
            })


class UploadParamValidator:

    requires_context = True

    def __call__(self, uploads_string, serializer_field):
        request = serializer_field.parent.context['request']

        uploads = uploads_string.split(';')

        for upload in uploads:
            try:
                resource_name, uri = upload.split(',')
            except ValueError as e:
                raise ValidationError({
                    'UPLOAD': 'Field is not of the form resource_name:URI'
                }) from e

            if uri.startswith('param:'):
                file_name = uri[len('param:'):]

                # check if alnum (+ underscore)
                if not (not file_name[0].isdigit() and all(c.isalnum() or c == '_' for c in file_name)):
                    raise ValidationError({
                        'UPLOAD': 'URI contains forbidden characters'
                    })

                # check if the file is in the body of the post request
                if file_name not in request.data:
                    raise ValidationError({
                        'UPLOAD': f'UPLOAD URI "{uri}" contains does not match uploaded file'
                    })

            elif uri.startswith('http:') or uri.startswith('https:'):
                URLValidator()(uri)

            else:
                raise ValidationError(f'UPLOAD URI "{uri}" is not supported')



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-16 14:17
from __future__ import unicode_literals

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_query', '0006_example'),
    ]

    operations = [
        migrations.AddField(
            model_name='queryjob',
            name='metadata',
            field=jsonfield.fields.JSONField(default={}),
            preserve_default=False,
        ),
    ]



from rest_framework.reverse import reverse

from django_user_agents.utils import get_user_agent


def get_job_url(request, kwargs):
    namespace = request.resolver_match.namespace
    base_name = request.resolver_match.url_name.rsplit('-', 1)[0]
    return reverse(f'{namespace}:{base_name}-detail', request=request, kwargs=kwargs)


def get_job_results(request, job):
    namespace = request.resolver_match.namespace
    base_name = request.resolver_match.url_name.rsplit('-', 1)[0]

    results = []
    for key in job.formats:
        url = reverse(f'{namespace}:{base_name}-result', args=[job.id, key])

        results.append({
            'result_type': key,
            'href': request.build_absolute_uri(url)
        })

    return results


def get_content_type(request, renderer):
    user_agent = get_user_agent(request)
    return 'application/xml' if user_agent.is_pc else renderer.media_type



# Generated by Django 3.2.10 on 2022-01-10 16:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0029_alter_ids'),
    ]

    operations = [
        migrations.AlterField(
            model_name='schema',
            name='license',
            field=models.CharField(blank=True, max_length=8, null=True, verbose_name='License'),
        ),
        migrations.AlterField(
            model_name='table',
            name='license',
            field=models.CharField(blank=True, max_length=8, null=True, verbose_name='License'),
        ),
    ]



from django.conf import settings
from django.core.exceptions import ValidationError
from django.urls import reverse

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.constants import ACCESS_LEVEL_PUBLIC
from daiquiri.core.utils import get_doi_url, import_class

from .constants import DATALINK_FIELDS, DATALINK_RELATION_TYPES
from .models import Datalink


def DatalinkAdapter():
    return import_class(settings.DATALINK_ADAPTER)()


class BaseDatalinkAdapter:
    """
    Each datalink adapter needs to configure a set of resource types.
    resource_types are declare as a list(string), i.e.: ['table', 'schema', ...]

    For each resource type, the following methods need to be implemented:

    * get_<resource_type>_list(self): returns a list of all resources for the resource_type

    * get_<resource_type>_identifier(self, resource): returns the identifier for
      a resource into a list (string)

    * get_<resource_type>_links(self, resource): returns a resource into a list
      of rows of the datalink table (list of python dicts)

    There are two further adapters, which do not declare resources:

    * DynamicDatalinkAdapter: the latter does not declare a resource, but will inject on the fly
      extra datalink entries according to the method: get_dyn_datalink_links()

    * QueryJobDatalinkAdapterMixin: The latter does not declare a resource either, but it injects
      extra context information for the datalink viewer.

    See the mixins below for an example.
    """

    resource_types = []

    def __init__(self):
        for resource_type in self.resource_types:
            for method in [f'get_{resource_type}_list',
                           f'get_{resource_type}_identifier',
                           f'get_{resource_type}_links']:
                if not hasattr(self, method):
                    message = f'\'{resource_type}\' is declared as resource_type, but \'{self.__class__.__name__}\' ' \
                               'object has no attribute \'{method}\''
                    raise NotImplementedError(message)

    def get_list(self):
        '''This is only used by rebuild_datalink_table, so it needs to gather only the tabular datalink entries.
        '''
        for resource_type in self.resource_types:
            yield from getattr(self, f'get_{resource_type}_list')()

    def get_identifier(self, resource_type, resource):
        return getattr(self, f'get_{resource_type}_identifier')(resource)

    def get_links(self, resource_type, resource):
        return getattr(self, f'get_{resource_type}_links')(resource)

    def get_context_data(self, request, **kwargs):
        '''Get the datalink related context data for a given request
        '''
        context = {}

        if 'ID' in kwargs:
            # more precise would be to use a serializer instead of list(QuerySet.values())
            context['datalinks'] = list(Datalink.objects.filter(ID=kwargs['ID']).order_by('semantics').values())
            context['ID'] = kwargs['ID']

        return context

    def get_datalink_rows(self, identifiers, **kwargs):
        '''Get the list of datalink entries for the provided identifiers (incl. table- and dynamic- datalink)
        '''
        # get the datalink entries from Datalink Table and metadata (Table and Schema)
        static_datalink_rows = list(Datalink.objects.filter(ID__in=identifiers).values())

        # get the dynamic datalink entries
        dyn_datalink_rows = self.get_dyn_datalink_links(identifiers)
        datalink_rows = static_datalink_rows + dyn_datalink_rows

        # create a full URI for the custom semantics
        for row in datalink_rows:
            if row['semantics'] in settings.DATALINK_CUSTOM_SEMANTICS:
                row['semantics'] = settings.SITE_URL + reverse('datalink:datalink-semantics') + row['semantics']

        field_names = [field['name'] for field in DATALINK_FIELDS]

        try:
            rows = [[link[key] for key in field_names] for link in datalink_rows]
        except KeyError as e:
            class_name = str(self.__class__)
            raise KeyError(f"The key '{e.args[0]}' is missing in one of the dictionaries returned by " \
                           f"{class_name}.get_dyn_datalink_links(id) or in the Datalink model.") from e
        except Exception as e:
            raise e

        # check for missing IDs and return error message
        for identifier in identifiers:
            if not any(filter(lambda row: row[0] == identifier, rows)):
                rows.append((identifier, None, None, f'NotFoundFault: {identifier}', None, None, None, None))

        return rows


class TablesDatalinkAdapterMixin:
    '''
    Gather the datalink entries from Release related Datalink tables (declared in settings.DATALINK_TABLES).
    '''

    tables = settings.DATALINK_TABLES

    def get_datalink_list(self):
        for table in self.tables:
            schema_name, table_name = table.split('.')
            for row in DatabaseAdapter().fetch_rows(schema_name, table_name, page_size=0):
                yield 'datalink', row

    def get_datalink_identifier(self, row):
        return row[1]

    def get_datalink_links(self, row):
        return [
            {
               'ID': self.get_datalink_identifier(row),
               'access_url': row[2] or '',
               'service_def': row[3] or '',
               'error_message': row[4] or '',
               'description': row[5] or '',
               'semantics': row[6],
               'content_type': row[7] or '',
               'content_length': row[8]
            }
        ]


class MetadataDatalinkAdapterMixin:
    '''
    Gather the documentation (metadata-page) and doi datalink entries from the metadata
    for each PUBLIC schemas and tables
    '''

    def get_schema_list(self):
        from daiquiri.metadata.models import Schema

        for schema in Schema.objects.iterator():
            yield 'schema', schema

    def get_schema_identifier(self, schema):
        return schema.name

    def get_schema_links(self, schema):
        schema_links = []

        if schema.metadata_access_level == ACCESS_LEVEL_PUBLIC:
            identifier = self.get_schema_identifier(schema)

            path = reverse('metadata:schema', args=[schema.name])
            access_url = settings.SITE_URL.rstrip('/') + path

            schema_links = [{
               'ID': identifier,
               'access_url': access_url,
               'service_def': '',
               'error_message': '',
               'description': f'Documentation for the {schema} schema',
               'semantics': '#documentation',
               'content_type': 'application/html',
               'content_length': None
            }]

            if schema.doi:
                schema_links.append({
                   'ID': identifier,
                   'access_url': get_doi_url(schema.doi),
                   'service_def': '',
                   'error_message': '',
                   'description': f'{schema.title}',
                   'semantics': '#doi',
                   'content_type': 'application/html',
                   'content_length': None
                })

            if schema.related_identifiers:
                for related_identifier in schema.related_identifiers:
                    access_url = related_identifier.get('related_identifier')
                    if access_url is not None:
                        description = DATALINK_RELATION_TYPES.get(related_identifier.get('relation_type'), '') \
                                                             .format(f'the {schema} schema') \
                                                             .capitalize()
                        if related_identifier.get('related_identifier_type') == "DOI":
                            access_url = get_doi_url(access_url)

                        schema_links.append({
                           'ID': identifier,
                           'access_url': access_url,
                           'service_def': '',
                           'error_message': '',
                           'description': description,
                           'semantics': '#auxiliary',
                           'content_type': 'application/html',
                           'content_length': None
                        })

        return schema_links

    def get_table_list(self):
        from daiquiri.metadata.models import Table

        for table in Table.objects.iterator():
            yield 'table', table

    def get_table_identifier(self, table):
        return f'{table.schema.name}.{table.name}'

    def get_table_links(self, table):
        table_links = []

        if table.metadata_access_level == ACCESS_LEVEL_PUBLIC and \
                table.schema.metadata_access_level == ACCESS_LEVEL_PUBLIC:
            identifier = self.get_table_identifier(table)

            path = reverse('metadata:table', args=[table.schema.name, table.name])
            access_url = settings.SITE_URL.rstrip('/') + path

            table_links.append({
               'ID': identifier,
               'access_url': access_url,
               'service_def': '',
               'error_message': '',
               'description': f'Documentation for the {table} table',
               'semantics': '#documentation',
               'content_type': 'application/html',
               'content_length': None
            })

            if table.doi:
                table_links.append({
                   'ID': identifier,
                   'access_url': get_doi_url(table.doi),
                   'service_def': '',
                   'error_message': '',
                   'description': f'{table.title}',
                   'semantics': '#doi',
                   'content_type': 'application/html',
                   'content_length': None
                })

            if table.related_identifiers:
                for related_identifier in table.related_identifiers:
                    access_url = related_identifier.get('related_identifier')
                    if access_url is not None:
                        description = DATALINK_RELATION_TYPES.get(related_identifier.get('relation_type'), '') \
                                                             .format(f'the {table} table') \
                                                             .capitalize()

                        if related_identifier.get('related_identifier_type') == "DOI":
                            access_url = get_doi_url(access_url)

                        table_links.append({
                           'ID': identifier,
                           'access_url': access_url,
                           'service_def': '',
                           'error_message': '',
                           'description': description,
                           'semantics': '#auxiliary',
                           'content_type': 'application/html',
                           'content_length': None
                        })

        return table_links


class DynamicDatalinkAdapterMixin:
    '''Define the interface to dynamically add datalink entries
    '''

    def get_dyn_datalink_links(self, IDs, **kwargs):
        '''No dynamically generated entries. Can be overwritten.

        this method should return a list of dict with the following keys:
             ID, access_url, service_def, error_message, description, semantics, content_type, content_length
        '''
        return([])

    def get_context_data(self, request, **kwargs):
        '''Inject dynamically generated Datalink entries into the context for the daiquiri.datalinks.views.datalink View
        '''
        context = super().get_context_data(request, **kwargs)

        if 'ID' in kwargs:
            context['datalinks'] = context['datalinks'] + self.get_dyn_datalink_links([kwargs['ID']], **kwargs)

        return context


class QueryJobDatalinkAdapterMixin:
    '''
    Injects the query job into the context data for the daiquiri.datalinks.views.datalink view
    '''

    def get_context_data(self, request, **kwargs):
        from daiquiri.query.models import QueryJob

        context = super().get_context_data(request, **kwargs)

        if 'job' in request.GET:
            try:
                context['query_job'] = QueryJob.objects.filter_by_owner(request.user).get(pk=request.GET['job'])
            except (QueryJob.DoesNotExist, ValidationError):
                pass

        return context

class DefaultDatalinkAdapter(DynamicDatalinkAdapterMixin,
                             MetadataDatalinkAdapterMixin,
                             TablesDatalinkAdapterMixin,
                             QueryJobDatalinkAdapterMixin,
                             BaseDatalinkAdapter):

    resource_types = ['datalink', 'schema', 'table']



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-21 08:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_jobs', '0008_job_error_summary'),
    ]

    operations = [
        migrations.AlterField(
            model_name='job',
            name='job_type',
            field=models.CharField(choices=[('QUERY', 'Query'), ('CUTOUT', 'Cutout')], max_length=10),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-06-02 13:27
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_contact', '0002_meta'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='contactmessage',
            options={'ordering': ('status', 'created', 'author'), 'permissions': (('view_contactmessage', 'Can view ContactMessage'),), 'verbose_name': 'Contact message', 'verbose_name_plural': 'Contact messages'},
        ),
    ]



from django.conf import settings
from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import gettext_lazy as _

from daiquiri.core.utils import get_date_display, import_class


class ContactMessage(models.Model):


    STATUS_ACTIVE = 'ACTIVE'
    STATUS_CLOSED = 'CLOSED'
    STATUS_SPAM = 'SPAM'
    STATUS_CHOICES = (
        (STATUS_ACTIVE, 'active'),
        (STATUS_CLOSED, 'closed'),
        (STATUS_SPAM, 'spam')
    )

    author = models.CharField(max_length=256)
    email = models.EmailField(max_length=256)
    subject = models.CharField(max_length=256)

    user = models.ForeignKey(User, blank=True, null=True, on_delete=models.SET_NULL, related_name='contact_messages')
    status = models.CharField(max_length=8, choices=STATUS_CHOICES)
    created = models.DateTimeField(blank=True, null=True)

    message = models.TextField()

    class Meta:
        ordering = ('-created', 'author')

        verbose_name = _('Contact message')
        verbose_name_plural = _('Contact messages')

    def __str__(self):
        return f"created={self.created}; email={self.email}; subject={self.subject}; status={self.status}"

    @property
    def status_label(self):
        return self.get_status_display()

    @property
    def created_label(self):
        return get_date_display(self.created)

    def set_status_closed(self):
        self.status = self.STATUS_CLOSED
        self.save()

    def set_status_active(self):
        self.status = self.STATUS_ACTIVE
        self.save()

    def set_status_spam(self):
        self.status = self.STATUS_SPAM
        self.save()



def MessageFilter():
    return import_class(settings.ANNOUNCEMENT_MESSAGE_FILTER)

class AnnouncementMessage(models.Model):

    # alert types must correspond to the bootstrap alert types
    ALERT_TYPE_INFO = "info"
    ALERT_TYPE_WARNING = "warning"
    ALERT_TYPE_URGENT = "danger"

    ANNOUNCEMENT_TYPE_CHOICES = (
        (ALERT_TYPE_INFO, "info"),
        (ALERT_TYPE_WARNING, "warning"),
        (ALERT_TYPE_URGENT, "urgent"),
    )

    title = models.CharField(  # noqa: DJ001
        max_length=100, blank=True, null=True,
        verbose_name=_("Title")
    )
    announcement = models.TextField(
        verbose_name=_("Announcement")
    )
    announcement_type = models.CharField(
        max_length=8,
        choices=ANNOUNCEMENT_TYPE_CHOICES,
        default=ALERT_TYPE_INFO,
        verbose_name=_("Announcement type")
    )
    updated = models.DateTimeField(
        auto_now=True,
        verbose_name=_("Updated")
    )
    visible = models.BooleanField(
        default=False,
        verbose_name=_("Visible")
    )
    visibility_filter = models.CharField(
        max_length=50, default="no_filter",
        verbose_name="Visibility filter"
    )

    class Meta:
        ordering = ('-updated', 'title')

        verbose_name = _('Announcement message')
        verbose_name_plural = _('Announcement messages')

    def __str__(self):
        return f"{self.title}: '{self.announcement}'"

    def get_filter(self):
        return getattr(MessageFilter(), str(self.visibility_filter))



import io

from django.utils.encoding import smart_str
from django.utils.xmlutils import SimplerXMLGenerator

from rest_framework.renderers import BaseRenderer


class XMLRenderer(BaseRenderer):

    media_type = 'application/xml'
    charset = 'utf-8'
    format = 'xml'

    def render(self, data, accepted_media_type=None, renderer_context=None):
        if data is None:
            return ''

        stream = io.StringIO()

        self.xml = SimplerXMLGenerator(stream, self.charset)
        self.xml.startDocument()
        self.render_document(data, accepted_media_type, renderer_context)
        self.xml.endDocument()
        return stream.getvalue()

    def render_document(self, data, accepted_media_type=None, renderer_context=None):
        raise NotImplementedError()

    def start(self, tag, attrs={}):
        self.xml.startElement(tag, {k: v for k, v in attrs.items() if v is not None})

    def end(self, tag):
        self.xml.endElement(tag)

    def node(self, tag, attrs, text=''):
        if text is None:
            attrs.update({'xsi:nil': 'true'})

        self.xml.startElement(tag, {k: str(v) for k, v in attrs.items() if v is not None})
        if text:
            self.xml.characters(smart_str(text))
        self.xml.endElement(tag)

    def _to_camel_case(self, snake_str):
        components = snake_str.split('_')
        return components[0] + "".join(x.title() for x in components[1:])


class VOTableRenderer(XMLRenderer):

    format = 'votable'

    root_attrs = {
        'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
        'xmlns': 'http://www.ivoa.net/xml/VOTable/v1.3',
        'xmlns:stc': 'http://www.ivoa.net/xml/STC/v1.30'
    }

    def render_document(self, data, accepted_media_type=None, renderer_context=None):
        self.start('VOTABLE', self.root_attrs)
        self.render_votable(data)
        self.end('VOTABLE')

    def render_votable(self, data, accepted_media_type=None, renderer_context=None):
        raise NotImplementedError()


class ErrorRenderer(VOTableRenderer):

    def get_error_string(self, data):
        if isinstance(data, dict):
            errors = []
            for parameter, parameter_errors in data.items():
                if isinstance(parameter_errors, (list, tuple)):
                    errors.append('{}: {}'.format(parameter, ', '.join([str(e) for e in parameter_errors])))
                else:
                    errors.append(f'{parameter}: {parameter_errors}')
            return '\n'.join(errors)
        elif isinstance(data, (list, tuple)):
            return '\n'.join(data)
        else:
            return data



import os
import shutil

from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('command', choices=[
            'all',
            'dist',
            'media',
            'npm',
            'python',
            'static',
        ])

    def handle(self, *args, **options):
        if options['command'] in ['all', 'dist']:
            self.clean_dir('dist')
            self.clean_dir('daiquiri.egg-info')
        if options['command'] in ['all', 'media']:
            self.clean_dir(settings.MEDIA_ROOT)
        if options['command'] in ['all', 'npm']:
            self.clean_dir('node_modules')
        if options['command'] in ['all', 'static']:
            self.clean_static()
        if options['command'] in ['all', 'python']:
            self.clean_python()

    def clean_python(self):
        for root, dirs, files in os.walk('.'):
            for dir_name in dirs:
                if dir_name == '__pycache__':
                    self.clean_dir(os.path.join(root, dir_name), quiet=True)

    def clean_static(self):
        self.clean_dir(settings.STATIC_ROOT)

        for path in [
            'daiquiri/auth/static/',
            'daiquiri/contact/static/',
            'daiquiri/core/static/',
            'daiquiri/metadata/static/',
            'daiquiri/query/static/',
            'daiquiri/serve/static/',
        ]:
            self.clean_dir(path)

    def clean_dir(self, path, quiet=False):
        if path and os.path.exists(path):
            shutil.rmtree(path)
            if not quiet:
                print(f'Directory "{path}" has been removed!')



from django.urls import path
from django.views.generic import TemplateView

from .views import authority, availability, capabilities, resource, web

app_name = 'registry'


urlpatterns = [
    path('', TemplateView.as_view(template_name='registry/root.html'), name='root'),
    path('resource', resource, name='resource'),
    path('availability', availability, name='availability'),
    path('capabilities', capabilities, name='capabilities'),
    path('authority/resource', authority, name='authority'),
    path('web/resource', web, name='web'),
]



from rest_framework import filters, viewsets
from rest_framework.authentication import SessionAuthentication, TokenAuthentication
from rest_framework.permissions import IsAuthenticated

from django_filters.rest_framework import DjangoFilterBackend

from daiquiri.core.paginations import ListPagination
from daiquiri.core.permissions import HasModelPermission
from daiquiri.core.viewsets import ChoicesViewSet

from .filters import SpamBackend
from .models import ContactMessage
from .serializers import ContactMessageSerializer


class ContactMessageViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)

    queryset = ContactMessage.objects.all()

    serializer_class = ContactMessageSerializer
    pagination_class = ListPagination

    filter_backends = (
        filters.SearchFilter,
        filters.OrderingFilter,
        DjangoFilterBackend,
        SpamBackend
    )
    search_fields = ('author', 'email', 'subject', 'message', 'status')
    filterset_fields = ('status', )


class StatusViewSet(ChoicesViewSet):
    permission_classes = (IsAuthenticated, )
    queryset = ContactMessage.STATUS_CHOICES



from rest_framework import filters

from .models import QueryJob


class JobFilterBackend(filters.BaseFilterBackend):

    def filter_queryset(self, request, queryset, view):

        phases = request.GET.getlist('phase')
        archived = request.GET.get('archived')

        if phases:
            queryset = queryset.filter(phase__in=phases)

        if archived is not None:
            if archived:
                queryset = queryset.filter(phase=QueryJob.PHASE_ARCHIVED)
            else:
                queryset = queryset.exclude(phase=QueryJob.PHASE_ARCHIVED)

        return queryset



import logging
import os

from django.conf import settings
from django.contrib.auth.views import redirect_to_login
from django.core.exceptions import PermissionDenied
from django.core.paginator import Paginator
from django.http import Http404
from django.shortcuts import render
from django.views.generic import View

from .search import Searcher
from .utils import get_directory, get_file_path, render_with_layout, send_file

logger = logging.getLogger(__name__)


class FileView(View):

    root = None

    def get(self, request, file_path, **kwargs):
        if self.root:
            logger.debug('root=%s', self.root)
            file_path = os.path.join(self.root, file_path)

        file_path = get_file_path(file_path)
        if file_path is None:
            logger.debug('%s not found', file_path)
            raise Http404

        directory = get_directory(request.user, file_path)
        if directory is None:
            logger.debug('%s is forbidden', file_path)
            if request.user.is_authenticated:
                raise PermissionDenied
            else:
                return redirect_to_login(request.path_info)

        if (file_path.endswith('.html') or file_path.endswith('.md')) and directory.layout:
            return render_with_layout(request, file_path)
        else:
            return send_file(request, file_path)


class SearchView(View):

    root = None

    def get(self, request, **kwargs):

        search_string = request.GET.get("q", "")

        results = Searcher.search_for_string(string_query=search_string)

        paginator = Paginator(results, settings.FILES_SEARCH_RESULTS_PER_PAGE)
        page_number = request.GET.get('page')
        search_results = paginator.get_page(page_number)

        context = {
                "search_results": search_results,
                "search_string": search_string,
                "num_of_search_results": len(results)
                }

        return render(request, "files/search-results.html", context)



from .mysql import MySQLAdapter


class MariaDBAdapter(MySQLAdapter):

    def build_query(self, schema_name, table_name, query, timeout, max_records):
        # construct the actual query
        params = {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name),
            'query': query,
            'timeout': timeout,
            'max_records': max_records
        }

        if max_records is not None:
            return 'SET STATEMENT max_statement_time={timeout} FOR CREATE TABLE {schema}.{table} ENGINE=ARIA ( {query} ) LIMIT {max_records};'.format(**params)  # noqa: E501
        else:
            return 'SET STATEMENT max_statement_time={timeout} FOR CREATE TABLE {schema}.{table} ENGINE=ARIA ( {query} );'.format(**params)  # noqa: E501

    def build_sync_query(self, query, timeout, max_records):
        # construct the actual query
        params = {
            'query': query,
            'timeout': timeout,
            'max_records': max_records
        }

        if max_records is not None:
            return 'SET STATEMENT max_statement_time={timeout} FOR {query} LIMIT {max_records};'.format(**params)
        else:
            return 'SET STATEMENT max_statement_time={timeout} FOR {query};'.format(**params)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-25 11:02
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0008_refactoring'),
    ]

    operations = [
        migrations.RenameField(
            model_name='column',
            old_name='size',
            new_name='arraysize',
        ),
    ]



from django.contrib.auth.models import AnonymousUser
from django.template import RequestContext, Template
from django.urls import reverse


def test_schema_menu(db, rf):
    template = "{% load metadata_tags %}{% schemas_menu %}"

    request = rf.get(reverse('home'))
    request.user = AnonymousUser()

    context = RequestContext(request, {})
    rendered_template = Template(template).render(context)

    assert 'daiquiri_data_obs' in rendered_template



STATS_RESOURCE_TYPES = [
    {
        'key': 'CONESEARCH',
        'label': 'Performed cone searches'
    },
    {
        'key': 'CUTOUT',
        'label': 'Performed cutouts'
    },
    {
        'key': 'FILE',
        'label': 'Downloaded static files'
    },
    {
        'key': 'CREATE_ZIP',
        'label': 'Created zip files'
    },
    {
        'key': 'CREATE_FILE',
        'label': 'Created files for download'
    },
    {
        'key': 'DOWNLOAD',
        'label': 'Downloaded files'
    },
    {
        'key': 'UPLOAD',
        'label': 'Uploaded files'
    },
    {
        'key': 'QUERY',
        'label': 'Queries'
    },
]



import os
from collections import OrderedDict

from django.conf import settings
from django.http import FileResponse, Http404
from django.utils.timezone import now

from rest_framework import filters, mixins, status, viewsets
from rest_framework.authentication import BasicAuthentication, SessionAuthentication, TokenAuthentication
from rest_framework.decorators import action
from rest_framework.exceptions import NotFound, ValidationError
from rest_framework.response import Response

from django_sendfile import sendfile

from daiquiri.core.paginations import ListPagination
from daiquiri.core.permissions import HasModelPermission
from daiquiri.core.utils import (
    filter_by_access_level,
    fix_for_json,
    get_client_ip,
    get_file_size,
    handle_file_upload,
    import_class,
)
from daiquiri.core.viewsets import ChoicesViewSet, RowViewSetMixin
from daiquiri.jobs.viewsets import AsyncJobViewSet, SyncJobViewSet
from daiquiri.stats.models import Record

from .filters import JobFilterBackend
from .models import DownloadJob, Example, QueryJob
from .permissions import HasPermission
from .serializers import (
    AsyncQueryJobSerializer,
    DownloadSerializer,
    DropdownSerializer,
    ExampleSerializer,
    FormDetailSerializer,
    FormListSerializer,
    QueryDownloadFormatSerializer,
    QueryJobCreateSerializer,
    QueryJobFormSerializer,
    QueryJobIndexSerializer,
    QueryJobRetrieveSerializer,
    QueryJobSerializer,
    QueryJobUpdateSerializer,
    QueryJobUploadSerializer,
    QueryLanguageSerializer,
    SyncQueryJobSerializer,
    UserExampleSerializer,
)
from .utils import (
    fetch_user_schema_metadata,
    get_download_config,
    get_format_config,
    get_quota,
    get_user_upload_directory,
    handle_upload_param,
    ingest_uploads,
)


class StatusViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )
    queryset = []

    def list(self, request):
        return Response({
            'guest': not request.user.is_authenticated,
            'queued_jobs': None,
            'size': QueryJob.objects.get_size(request.user),
            'hash':  QueryJob.objects.get_hash(request.user),
            'quota': get_quota(request.user),
            'upload_limit': get_quota(request.user, quota_settings='QUERY_UPLOAD_LIMIT')
        })


class FormViewSet(mixins.ListModelMixin, mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )

    def get_queryset(self):
        return settings.QUERY_FORMS

    def get_object(self):
        try:
            return next(form for form in self.get_queryset() if form.get('key') == self.kwargs.get('pk'))
        except StopIteration as e:
            raise Http404 from e

    def get_serializer_class(self):
        if self.action == 'list':
            return FormListSerializer
        else:
            return FormDetailSerializer



class DropdownViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )

    serializer_class = DropdownSerializer

    def get_queryset(self):
        return settings.QUERY_DROPDOWNS


class DownloadViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )

    serializer_class = DownloadSerializer

    def get_queryset(self):
        return settings.QUERY_DOWNLOADS


class QueryJobViewSet(RowViewSetMixin, viewsets.ModelViewSet):
    permission_classes = (HasPermission, )
    authentication_classes = (SessionAuthentication, BasicAuthentication, TokenAuthentication)
    pagination_class = ListPagination

    filter_backends = (
        filters.SearchFilter,
        filters.OrderingFilter,
        JobFilterBackend
    )
    search_fields = ('id', 'run_id', 'table_name', 'phase')
    filter_fields = ('phase', )

    def get_queryset(self):
        queryset = QueryJob.objects.filter_by_owner(self.request.user).order_by('-creation_time')

        # hide TAP queries for the anonymous user
        if self.request.user.is_anonymous:
            queryset = queryset.filter(job_type=QueryJob.JOB_TYPE_INTERFACE)

        return queryset

    def get_serializer_class(self):
        if self.action == 'list' or self.action == 'retrieve' or self.action == 'abort':
            return QueryJobRetrieveSerializer
        elif self.action == 'create':
            return QueryJobCreateSerializer
        elif self.action == 'update' or self.action == 'partial_update':
            return QueryJobUpdateSerializer
        else:
            return QueryJobSerializer

    def get_throttles(self):
        if self.action == 'create':
            self.throttle_scope = 'query.create'

        return super().get_throttles()

    def perform_create(self, serializer):
        job = QueryJob(
            job_type=QueryJob.JOB_TYPE_INTERFACE,
            owner=(None if self.request.user.is_anonymous else self.request.user),
            run_id=serializer.data.get('run_id'),
            table_name=serializer.data.get('table_name'),
            query_language=serializer.data.get('query_language'),
            query=serializer.data.get('query'),
            queue=serializer.data.get('queue'),
            client_ip=get_client_ip(self.request)
        )
        job.process()
        job.save()
        job.run()

        # inject the job id into the serializers data
        serializer._data['id'] = job.id

    def perform_update(self, serializer):
        serializer.save()

    def perform_destroy(self, instance):
        instance.archive()

    @action(detail=False, methods=['get'])
    def index(self, request):
        queryset = self.get_queryset().exclude(phase=QueryJob.PHASE_ARCHIVED)[:1000]
        serializer = QueryJobIndexSerializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=False, methods=['get'])
    def tables(self, request):
        queryset = self.get_queryset().filter(phase=QueryJob.PHASE_COMPLETED)[:100]
        return Response(fetch_user_schema_metadata(request.user, queryset))

    @action(detail=False, methods=['post'], url_path='forms/(?P<form_key>[a-z]+)', url_name='forms')
    def forms(self, request, form_key):
        # follows CreateModelMixin.create(request, *args, **kwargs)
        serializer = QueryJobFormSerializer(
            data=request.data,
            form_key=form_key,
            context=self.get_serializer_context()
        )
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(detail=False, methods=['post'], url_path='upload', url_name='upload')
    def upload(self, request):

        if not settings.QUERY_UPLOAD:
            raise Http404

        serializer = QueryJobUploadSerializer(data=request.data, context={
            'request': request,
            'view': self
        })
        serializer.is_valid(raise_exception=True)

        file_name = handle_file_upload(get_user_upload_directory(self.request.user), serializer.validated_data['file'])

        job = QueryJob(
            job_type=QueryJob.JOB_TYPE_INTERFACE,
            owner=(None if self.request.user.is_anonymous else self.request.user),
            run_id=serializer.validated_data.get('run_id'),
            table_name=serializer.validated_data.get('table_name'),
            client_ip=get_client_ip(self.request)
        )
        job.process(upload=True)
        job.save()
        job.ingest(file_name)

        serializer = QueryJobRetrieveSerializer(instance=job)
        return Response(serializer.data)

    @action(detail=True, methods=['put'])
    def abort(self, request, pk=None):
        try:
            job = self.get_queryset().get(pk=pk)
            job.abort()

            serializer = QueryJobRetrieveSerializer(instance=job)
            return Response(serializer.data)
        except QueryJob.DoesNotExist as e:
            raise Http404 from e

    @action(detail=True, methods=['get'])
    def rows(self, request, pk=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist as e:
            raise NotFound from e

        # get the row query params from the request
        ordering, page, page_size, search, filters = self._get_query_params(job.columns())

        # get column names from the request
        column_names = self.request.GET.getlist('column')

        # get the count and the rows from the job
        count, results = job.rows(column_names, ordering, page, page_size, search, filters)

        # return ordered dict to be send as json
        return Response(OrderedDict((
            ('count', count),
            ('results', fix_for_json(results)),
            ('next', self._get_next_url(page, page_size, count)),
            ('previous', self._get_previous_url(page))
        )))

    @action(detail=True, methods=['get'])
    def columns(self, request, pk=None, format_key=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist as e:
            raise NotFound from e

        return Response(job.columns())

    @action(detail=True, methods=['get'], url_name='submitted-downloads',
            url_path=r'downloads')
    def get_downloads(self, request, pk=None):
        try:
            self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist as e:
            raise NotFound from e

        response = []
        for download_key in [query_download['key'] for query_download in settings.QUERY_DOWNLOADS]:
            download_config = get_download_config(download_key)
            if download_config is None:
                raise ValidationError({'download': f'Download key "{download_key}" is not supported.'})

            download_job_model = import_class(download_config['model'])
            download_jobs = download_job_model.objects.filter(query_job=pk)
            for download_job in download_jobs:
                job_response = {
                    'key': download_key,
                    'id': download_job.id,
                    'phase': download_job.phase,
                    'error_summary': download_job.error_summary,
                    'size': get_file_size(download_job.file_path),
                }
                for par in download_config.get('params', []):
                    if hasattr(download_job, par):
                        job_response[par] = getattr(download_job, par)
                response.append(job_response)


        return Response(response)


    @action(detail=True, methods=['get'], url_name='download',
            url_path=r'download/(?P<download_key>[a-z\-]+)/(?P<download_job_id>[A-Za-z0-9\-]+)')
    def download(self, request, pk=None, download_key=None, download_job_id=None):
        try:
            self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist as e:
            raise NotFound from e

        download_config = get_download_config(download_key)
        if download_config is None:
            raise ValidationError({'download': f'Download key "{download_key}" is not supported.'})

        download_job_model = import_class(download_config['model'])

        try:
            download_job = download_job_model.objects.get(query_job=pk, pk=download_job_id)
        except download_job_model.DoesNotExist as e:
            raise NotFound from e

        if download_job.phase == download_job.PHASE_COMPLETED and request.GET.get('download', False):
            Record.objects.create(
                time=now(),
                resource_type='DOWNLOAD',
                resource={
                    'job_id': download_job.id,
                    'job_type': download_job.job_type,
                    'file_path': download_job.file_path
                },
                client_ip=download_job.client_ip,
                user=download_job.owner,
                size=os.path.getsize(download_job.file_path)
            )
            return sendfile(request, download_job.file_path, attachment=True)
        else:
            return Response({
                'phase': download_job.phase,
                'size': get_file_size(download_job.file_path),
            })

    @action(detail=True, methods=['post'], url_name='create-download',
            url_path=r'download/(?P<download_key>[a-z\-]+)')
    def create_download(self, request, pk=None, download_key=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist as e:
            raise NotFound from e

        download_config = get_download_config(download_key)
        if download_config is None:
            raise ValidationError({'download': f'Download key "{download_key}" is not supported.'})

        download_job_model = import_class(download_config['model'])

        params = {param: request.data.get(param) for param in download_config.get('params', [])}

        try:
            download_job = download_job_model.objects.get(query_job=job, **params)
        except download_job_model.DoesNotExist:
            download_job = download_job_model(
                job_type=QueryJob.JOB_TYPE_INTERFACE,
                owner=(None if self.request.user.is_anonymous else self.request.user),
                client_ip=get_client_ip(self.request),
                query_job=job,
                **params
            )
            download_job.save()

        # check if the file was lost
        # and allow re-run the aborted jobs and jobs exited with an error
        if (download_job.phase == download_job.PHASE_COMPLETED and \
                not os.path.isfile(download_job.file_path)) or \
                download_job.phase == download_job.PHASE_ERROR or \
                download_job.phase == download_job.PHASE_ABORTED:
            download_job.phase = download_job.PHASE_PENDING
            download_job.save()

        if download_job.phase == download_job.PHASE_PENDING:
            try:
                download_job.process()
            except ValidationError as e:
                download_job.phase = download_job.PHASE_ERROR
                download_job.error_summary = str(e)
                download_job.save()
                raise e

            download_job.save()
            download_job.run()

        return Response({
            'id': download_job.id,
            'key': download_key
        })


    @action(detail=True, methods=['get'], url_path=r'stream/(?P<format_key>[A-Za-z0-9\-]+)', url_name='stream')
    def stream(self, request, pk=None, format_key=None):
        try:
            job = self.get_queryset().get(pk=pk)
        except QueryJob.DoesNotExist as e:
            raise NotFound from e

        try:
            format_config = get_format_config(format_key)
        except IndexError as e:
            raise ValidationError({'format': "Not supported."}) from e

        try:
            download_job = DownloadJob.objects.get(query_job=job, format_key=format_key)

            # check if the file was lost
            if download_job.phase == download_job.PHASE_COMPLETED and os.path.isfile(download_job.file_path):
                # stream the previously created file
                return sendfile(request, download_job.file_path)
        except DownloadJob.DoesNotExist:
            pass

        # stream the table directly from the database
        response = FileResponse(job.stream(format_key), content_type=format_config['content_type'])
        return response


class ExampleViewSet(viewsets.ModelViewSet):
    permission_classes = (HasModelPermission, )
    serializer_class = ExampleSerializer
    pagination_class = ListPagination
    queryset = Example.objects.all()

    filter_backends = (
        filters.SearchFilter,
        filters.OrderingFilter
    )
    search_fields = ('name', 'description', 'query_string')

    @action(detail=False, methods=['get'], permission_classes=(HasPermission, ))
    def user(self, request):
        examples = Example.objects.filter_by_access_level(self.request.user)
        serializer = UserExampleSerializer(examples, many=True)
        return Response(serializer.data)


class QueueViewSet(ChoicesViewSet):
    permission_classes = (HasPermission, )

    def get_queryset(self):
        items = filter_by_access_level(self.request.user, settings.QUERY_QUEUES)
        return [(item['key'], item['label']) for item in items]


class QueryLanguageViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )
    serializer_class = QueryLanguageSerializer

    def get_queryset(self):
        return filter_by_access_level(self.request.user, settings.QUERY_LANGUAGES)


class QueryDownloadFormatViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    permission_classes = (HasPermission, )
    serializer_class = QueryDownloadFormatSerializer

    def get_queryset(self):
        return settings.QUERY_DOWNLOAD_FORMATS


class PhaseViewSet(ChoicesViewSet):
    permission_classes = (HasPermission, )
    authentication_classes = (SessionAuthentication, TokenAuthentication)
    queryset = QueryJob.PHASE_CHOICES


class SyncQueryJobViewSet(SyncJobViewSet):
    permission_classes = (HasPermission, )
    serializer_class = SyncQueryJobSerializer

    parameter_map = {
        'FORMAT': 'response_format',
        'TABLE_NAME': 'table_name',
        'LANG': 'query_language',
        'QUERY': 'query'
    }

    def get_queryset(self):
        return QueryJob.objects.filter_by_owner(self.request.user)

    def handle_upload(self, job, upload_string):
        job.uploads = handle_upload_param(self.request, upload_string)
        ingest_uploads(job.uploads, job.owner)


class AsyncQueryJobViewSet(AsyncJobViewSet):
    permission_classes = (HasPermission, )
    serializer_class = AsyncQueryJobSerializer

    parameter_map = {
        'FORMAT': 'response_format',
        'TABLE_NAME': 'table_name',
        'LANG': 'query_language',
        'QUEUE': 'queue',
        'QUERY': 'query'
    }

    def get_queryset(self):
        return QueryJob.objects.filter_by_owner(self.request.user)

    def handle_upload(self, job, upload_string):
        job.uploads = handle_upload_param(self.request, upload_string)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-02-23 13:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_metadata', '0018_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='column',
            name='index_for',
            field=models.CharField(blank=True, help_text='The columns which this column is an index for (e.g. for pgSphere).', max_length=256, null=True, verbose_name='Index for'),
        ),
    ]



# Generated by Django 3.2.10 on 2022-01-22 13:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_files', '0006_alter_directory_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='directory',
            name='layout',
            field=models.BooleanField(default=True, help_text='Use the page layout with the content.', verbose_name='Layout'),
        ),
    ]



from datetime import datetime

from django.apps import apps
from django.conf import settings
from django.utils.timezone import now

from daiquiri.core.adapter import DatabaseAdapter
from daiquiri.core.constants import ACCESS_LEVEL_PUBLIC
from daiquiri.core.utils import get_doi, import_class
from daiquiri.datalink.adapter import DatalinkAdapter


def OaiAdapter():
    return import_class(settings.OAI_ADAPTER)()


class BaseOaiAdapter:
    """
    Each OAI adapter needs to configure a list of resource_types.

    For each resource_type, the following methods need to be implemented:

    * get_<resource_type>_list(self): returns a list of all resources for this resource_type

    * get_<resource_type>(self, resource_id): returns a resources for a given pk

    * get_<resource_type>_record(self, resource): returns the oai record for a given resource

    In addition the following attributes need to be set:

    * <resource_type>_metadata_prefixes: a list of metadata prefixes for this resource_type

    * <metadata_prefix>_<resource_type>_serializer_class: a serializer class for each
      metadata prefix and resource_type

    Instead of these attributes, getter functions can be implemented,
    e.g. get_<resource_type>_metadata_prefixes(self)

    See the mixins below for an example.
    """

    identifier_schema = 'oai'
    identifier_repository = settings.SITE_IDENTIFIER
    identifier_delimiter = ':'

    resource_types = []

    def __init__(self):
        for resource_type in self.resource_types:
            for method in [f'get_{resource_type}_list',
                           f'get_{resource_type}',
                           f'get_{resource_type}_record']:
                if not hasattr(self, method):
                    message = f'\'{resource_type}\' is declared as resource_type, but \'{self.__class__.__name__}\' ' \
                               'object has no attribute \'{method}\''
                    raise NotImplementedError(message)

            for metadata_prefix in self.get_metadata_prefixes(resource_type):
                self.get_serializer_class(metadata_prefix, resource_type)

    def get_resource_list(self):
        for resource_type in self.resource_types:
            yield from getattr(self, f'get_{resource_type}_list')()

    def get_resource(self, record):
        return getattr(self, f'get_{record.resource_type}')(record.resource_id)

    def get_record(self, resource_type, resource):
        return getattr(self, f'get_{resource_type}_record')(resource)

    def get_metadata_prefixes(self, resource_type):
        metadata_prefixes_attribute = f'{resource_type}_metadata_prefixes'

        if hasattr(self, metadata_prefixes_attribute):
            return getattr(self, metadata_prefixes_attribute)
        else:
            return getattr(self, f'get_{metadata_prefixes_attribute}')()

    def get_serializer_class(self, metadata_prefix, resource_type):
        serializer_class_attribute = f'{metadata_prefix}_{resource_type}_serializer_class'
        serializer_class_attribute_method = f'get_{serializer_class_attribute}'

        if hasattr(self, serializer_class_attribute):
            return getattr(self, serializer_class_attribute)
        elif hasattr(self, serializer_class_attribute_method):
            return getattr(self, serializer_class_attribute_method)()
        else:
            message = f'\'{resource_type}\' is declared as resource_type, but \'{self.__class__.__name__}\' ' \
                       'object has no attribute \'{serializer_class_attribute_method}\''
            raise NotImplementedError(message)

    def get_renderer(self, metadata_prefix):
        renderer_class = next(metadata_format['renderer_class']
                              for metadata_format in settings.OAI_METADATA_FORMATS
                              if metadata_format['prefix'] == metadata_prefix)
        return import_class(renderer_class)()

    def get_identifier(self, identifier):
        if identifier is None:
            return None

        if self.identifier_repository:
            return self.identifier_delimiter.join([self.identifier_schema, self.identifier_repository, identifier])
        else:
            return self.identifier_delimiter.join([self.identifier_schema, identifier])

    def get_sample_identifier(self):
        if self.identifier_repository:
            return self.identifier_delimiter.join([self.identifier_schema, self.identifier_repository, 'example'])
        else:
            return self.identifier_delimiter.join([self.identifier_schema, 'example'])


class MetadataOaiAdapterMixin:

    schema_metadata_prefixes = ('oai_dc', 'oai_datacite', 'datacite')
    table_metadata_prefixes = ('oai_dc', 'oai_datacite', 'datacite')

    def get_oai_dc_schema_serializer_class(self):
        return import_class('daiquiri.metadata.serializers.dublincore.DublincoreSchemaSerializer')

    def get_oai_dc_table_serializer_class(self):
        return import_class('daiquiri.metadata.serializers.dublincore.DublincoreTableSerializer')

    def get_oai_datacite_schema_serializer_class(self):
        return import_class('daiquiri.metadata.serializers.datacite.DataciteSchemaSerializer')

    def get_oai_datacite_table_serializer_class(self):
        return import_class('daiquiri.metadata.serializers.datacite.DataciteTableSerializer')

    def get_datacite_schema_serializer_class(self):
        return import_class('daiquiri.metadata.serializers.datacite.DataciteSchemaSerializer')

    def get_datacite_table_serializer_class(self):
        return import_class('daiquiri.metadata.serializers.datacite.DataciteTableSerializer')

    def get_schema_list(self):
        from daiquiri.metadata.models import Schema

        for schema in Schema.objects.iterator():
            yield 'schema', schema

    def get_table_list(self):
        from daiquiri.metadata.models import Table

        for table in Table.objects.iterator():
            yield 'table', table

    def get_schema(self, pk):
        from daiquiri.metadata.models import Schema

        try:
            schema = Schema.objects.get(pk=pk)
            return self.update_metadata(schema)
        except Schema.DoesNotExist:
            return None

    def get_table(self, pk):
        from daiquiri.metadata.models import Table

        try:
            table = Table.objects.get(pk=pk)
            return self.update_metadata(table)
        except Table.DoesNotExist:
            return None

    def update_metadata(self, instance):
        if hasattr(self, 'get_datalink'):
            for key, values in self.get_datalink(str(instance)).items():
                try:
                    attr = getattr(instance, key)
                    if isinstance(attr, dict):
                        attr.update(values)
                    elif isinstance(attr, list):
                        attr += values
                    else:
                        attr = values
                except AttributeError:
                    attr = values

                # update the instance
                setattr(instance, key, attr)

        return instance

    def get_schema_record(self, schema):
        identifier = self.get_identifier(f'schemas/{schema}')
        datestamp = schema.updated or schema.published or now().date()
        set_spec = 'schemas'
        public = (schema.metadata_access_level == ACCESS_LEVEL_PUBLIC) \
            and (schema.published is not None)

        return schema.pk, identifier, datestamp, set_spec, public

    def get_table_record(self, table):
        identifier = self.get_identifier(f'tables/{table}')
        datestamp = table.updated or table.published or now().date()
        set_spec = 'tables'
        public = (table.metadata_access_level == ACCESS_LEVEL_PUBLIC) \
            and (table.published is not None) \
            and (table.schema.metadata_access_level == ACCESS_LEVEL_PUBLIC) \
            and (table.schema.published is not None)

        return table.pk, identifier, datestamp, set_spec, public


class DatalinkOAIAdapterMixin:

    tables = settings.DATALINK_TABLES

    datalink_metadata_prefixes = ('oai_dc', 'oai_datacite', 'datacite')

    def get_oai_dc_datalink_serializer_class(self):
        return import_class('daiquiri.datalink.serializers.DublincoreSerializer')

    def get_oai_datacite_datalink_serializer_class(self):
        return import_class('daiquiri.datalink.serializers.DataciteSerializer')

    def get_datacite_datalink_serializer_class(self):
        return import_class('daiquiri.datalink.serializers.DataciteSerializer')

    def get_datalink_list(self):
        '''This function is used by rebuild_oai_schema only, it only needs to gather
           the doi objects declared via datalink (no other entries).
        '''
        for table in self.tables:
            schema_name, table_name = table.split('.')
            rows = DatabaseAdapter().fetch_rows(schema_name, table_name, column_names=['ID', 'access_url'],
                                                page_size=0, filters={'semantics': '#doi'})

            for ID, access_url in rows:
                yield 'datalink', {'id': str(ID), 'doi': get_doi(access_url)}

    def get_datalink(self, pk):

        adapter = DatalinkAdapter()
        # get all datalink entries: DatalinkTables, Metadata and Dynamic
        rows = adapter.get_datalink_rows([pk])

        datalink = {
            'formats': [],
            'alternate_identifiers': [
                {
                    'alternate_identifier': pk,
                    'alternate_identifier_type': 'datalink'
                }
            ],
            'related_identifiers': []
        }
        for _, access_url, _, _, description, semantics, content_type, content_length in rows:
            # doi is a custom datalink semantic which means that it will contain the full URL to the description
            # hence, only the end of the semantics string should be checked for #doi
            if semantics.endswith('#doi'):
                datalink['doi'] = get_doi(access_url)
                datalink['title'] = description

            elif semantics == '#this':
                if content_type not in datalink['formats']:
                    datalink['formats'].append(content_type)
                datalink['related_identifiers'].append({
                    'related_identifier': access_url,
                    'related_identifier_type': 'URL',
                    'relation_type': 'IsDescribedBy'
                })

            elif semantics == '#detached-header':
                if content_type not in datalink['formats']:
                    datalink['formats'].append(content_type)
                datalink['related_identifiers'].append({
                    'related_identifier': access_url,
                    'related_identifier_type': 'URL',
                    'relation_type': 'IsSupplementedBy'
                })

            elif semantics == '#documentation':
                datalink['related_identifiers'].append({
                    'related_identifier': access_url,
                    'related_identifier_type': 'URL',
                    'relation_type': 'IsDocumentedBy'
                })

            elif semantics == '#progenitor':
                datalink['related_identifiers'].append({
                    'related_identifier': access_url,
                    'related_identifier_type': 'URL',
                    'relation_type': 'IsDerivedFrom'
                })

            elif semantics == '#preview':
                datalink['alternate_identifiers'].append({
                    'alternate_identifier': access_url,
                    'alternate_identifier_type': 'DOI Landing Page'
                })

            elif semantics == '#preview-image':
                datalink['related_identifiers'].append({
                    'related_identifier': access_url,
                    'related_identifier_type': 'URL',
                    'relation_type': 'IsSupplementedBy'
                })

            elif semantics == '#auxiliary':
                datalink['related_identifiers'].append({
                    'related_identifier': access_url,
                    'related_identifier_type': 'URL',
                    'relation_type': 'References'
                })

        return datalink

    def get_datalink_record(self, datalink):
        if '/' in datalink['id']:
            # follow the convention <set_spec>/<identifier>
            identifier = self.get_identifier(datalink['id'])
            set_spec = datalink['id'].split('/')[0]
        else:
            identifier = self.get_identifier('datalinks/{}'.format(datalink['id']))
            set_spec = 'datalinks'

        datestamp = datetime.strptime(settings.SITE_CREATED, '%Y-%m-%d').date()
        public = True

        return datalink['id'], identifier, datestamp, set_spec, public


class RegistryOaiAdapterMixin:

    # services to appear in the registry oai records
    # does not need to be customized, active apps will be discovered automatically
    services = [
        'registry',
        'authority',
        'web',
        'tap',
        'conesearch',
        'datalink'
    ]

    service_metadata_prefixes = ['oai_dc', 'ivo_vor']

    def get_oai_dc_service_serializer_class(self):
        return import_class('daiquiri.registry.serializers.DublincoreSerializer')

    def get_ivo_vor_service_serializer_class(self):
        return import_class('daiquiri.registry.serializers.VoresourceSerializer')

    def get_service_list(self):
        for pk in self.services:
            service = self.get_service(pk)
            if service is not None:
                yield 'service', service

    def get_service(self, pk):
        if pk in self.services:
            return getattr(self, f'get_{pk}_service')()

    def get_service_record(self, service):
        identifier = service['identifier']
        datestamp = settings.SITE_UPDATED
        set_spec = 'ivo_managed'
        public = True

        return service['service'], identifier, datestamp, set_spec, public

    def get_registry_service(self):
        if apps.is_installed('daiquiri.registry'):
            from daiquiri.registry.vo import get_resource
            return get_resource()

    def get_authority_service(self):
        if apps.is_installed('daiquiri.registry'):
            from daiquiri.registry.vo import get_authority_resource
            return get_authority_resource()

    def get_web_service(self):
        if apps.is_installed('daiquiri.registry'):
            from daiquiri.registry.vo import get_web_resource
            return get_web_resource()

    def get_tap_service(self):
        if apps.is_installed('daiquiri.tap'):
            from daiquiri.tap.vo import get_resource
            return get_resource()

    def get_conesearch_service(self):
        if apps.is_installed('daiquiri.conesearch'):
            from daiquiri.conesearch.vo import get_resource
            return get_resource()

    def get_datalink_service(self):
        if apps.is_installed('daiquiri.datalink'):
            from daiquiri.datalink.vo import get_resource
            return get_resource()


class DefaultOaiAdapter(RegistryOaiAdapterMixin,
                        MetadataOaiAdapterMixin,
                        DatalinkOAIAdapterMixin,
                        BaseOaiAdapter):

    resource_types = ['service', 'schema', 'table', 'datalink']



import socket

from django.conf import settings
from django.core.management.base import BaseCommand

COMMANDS = {
    'django.db.backends.mysql': {
        'default': '''
-- Run the following commands on \'%(HOST)s\':
CREATE USER \'%(USER)s\'@\'%(CLIENT)s\' identified by \'%(PASSWORD)s\';
CREATE DATABASE `%(NAME)s` CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;
GRANT ALL PRIVILEGES ON `%(NAME)s`.* to \'%(USER)s\'@\'%(CLIENT)s\';
''',
        'data': '''
-- Run the following commands on \'%(HOST)s\':
CREATE USER \'%(USER)s\'@\'%(CLIENT)s\' identified by \'%(PASSWORD)s\';
CREATE DATABASE `%(NAME)s` CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;
CREATE DATABASE `%(TAP_SCHEMA)s` CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;
CREATE DATABASE `%(TAP_UPLOAD)s` CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;
CREATE DATABASE `%(OAI_SCHEMA)s` CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci;
GRANT SELECT ON `%(NAME)s`.* to \'%(USER)s\'@\'%(CLIENT)s\';
GRANT ALL PRIVILEGES ON `%(TAP_SCHEMA)s`.* to \'%(USER)s\'@\'%(CLIENT)s\';
GRANT ALL PRIVILEGES ON `%(TAP_UPLOAD)s`.* to \'%(USER)s\'@\'%(CLIENT)s\';
GRANT ALL PRIVILEGES ON `%(OAI_SCHEMA)s`.* to \'%(USER)s\'@\'%(CLIENT)s\';
GRANT ALL PRIVILEGES ON `%(PREFIX)s%%`.* to \'%(USER)s\'@\'%(CLIENT)s\';
''',
        'schema': '''
-- Run the following commands on \'%(HOST)s\':
GRANT SELECT ON `%(SCHEMA_NAME)s`.* TO \'%(USER)s\'@\'%(CLIENT)s\';
''',
        'datalink': '''
-- Run the following commands on %(HOST)s:
CREATE TABLE %(TABLE_NAME)s (
  `datalink_id` bigint(20) NOT NULL AUTO_INCREMENT,
  `ID` varchar(256) NOT NULL,
  `access_url` varchar(256),
  `service_def` varchar(80),
  `error_message` varchar(256),
  `description` varchar(256),
  `semantics` varchar(80) NOT NULL,
  `content_type` varchar(80),
  `content_length` bigint(20),
  PRIMARY KEY (`link_id`)
);
'''
    },
    'django.db.backends.postgresql': {
        'default': '''
-- Run the following commands on \'%(HOST)s\':
CREATE USER %(USER)s WITH PASSWORD \'%(PASSWORD)s\';
CREATE DATABASE %(NAME)s WITH OWNER %(USER)s;
''',
        'data': '''
-- Run the following commands on \'%(HOST)s\':
CREATE USER %(USER)s WITH PASSWORD \'%(PASSWORD)s\';
CREATE DATABASE %(NAME)s;
GRANT CREATE ON DATABASE %(NAME)s TO %(USER)s;
\\c %(NAME)s
CREATE SCHEMA %(TAP_SCHEMA)s AUTHORIZATION %(USER)s;
CREATE SCHEMA %(TAP_UPLOAD)s AUTHORIZATION %(USER)s;
CREATE SCHEMA %(OAI_SCHEMA)s AUTHORIZATION %(USER)s;
''',
        'schema': '''
-- Run the following commands on %(HOST)s:
\\c %(NAME)s
GRANT USAGE ON SCHEMA %(SCHEMA_NAME)s TO %(USER)s;
GRANT SELECT ON ALL TABLES IN SCHEMA %(SCHEMA_NAME)s TO %(USER)s;
''',
        'datalink': '''
-- Run the following commands on %(HOST)s:
CREATE TABLE %(TABLE_NAME)s (
    datalink_id serial PRIMARY KEY,
    "ID" character varying(256) NOT NULL,
    access_url character varying(256),
    service_def character varying(80),
    error_message character varying(256),
    description character varying(256),
    semantics character varying(80) NOT NULL,
    content_type character varying(80),
    content_length bigint
);
'''
    }
}


class Command(BaseCommand):

    requires_system_checks = []
    can_import_settings = True

    def add_arguments(self, parser):
        parser.add_argument('--schema', help='Show commands for a science schema.')
        parser.add_argument('--datalink', help='Show commands for a datalink table.')

    def get_config(self, key):
        config = settings.DATABASES.get(key)

        if config:
            if 'HOST' not in config or not config['HOST'] or config['HOST'] in ['localhost', '127.0.0.1', '::1']:
                config['HOST'] = 'localhost'
                config['CLIENT'] = 'localhost'
            else:
                config['CLIENT'] = socket.gethostname()

            config['PREFIX'] = settings.QUERY_USER_SCHEMA_PREFIX
            config['TAP_SCHEMA'] = settings.TAP_SCHEMA
            config['TAP_UPLOAD'] = settings.TAP_UPLOAD
            config['OAI_SCHEMA'] = settings.OAI_SCHEMA

        return config

    def handle(self, *args, **options):
        if options['schema']:
            config = self.get_config('data')
            config['SCHEMA_NAME'] = options['schema']
            print(COMMANDS[config['ENGINE']]['schema'] % config)
        elif options['datalink']:
            config = self.get_config('data')
            config['TABLE_NAME'] = options['datalink']
            print(COMMANDS[config['ENGINE']]['datalink'] % config)
        else:
            for key in ['default', 'data']:
                config = self.get_config(key)
                if config:
                    print(COMMANDS[config['ENGINE']][key] % config)



from django.core.management.base import BaseCommand

from daiquiri.datalink.adapter import DatalinkAdapter
from daiquiri.datalink.models import Datalink


class Command(BaseCommand):

    def handle(self, *args, **options):

        adapter = DatalinkAdapter()

        Datalink.objects.all().delete()

        datalinks = []
        for resource_type, resource in adapter.get_list():
            links = adapter.get_links(resource_type, resource)
            for link in links:
                datalinks.append(Datalink(**link))

        Datalink.objects.bulk_create(datalinks)



from django.conf import settings

from .models import Column, Schema, Table


def get_user_columns(user, schema_name, table_name):
    # check permissions on the schema
    try:
        schema = Schema.objects.filter_by_access_level(user).get(name=schema_name)
    except Schema.DoesNotExist:
        return []

    # check permissions on the table
    try:
        table = Table.objects.filter_by_access_level(user).filter(schema=schema).get(name=table_name)
    except Table.DoesNotExist:
        return []

    # get columns for this table
    if settings.METADATA_COLUMN_PERMISSIONS:
        return Column.objects.filter_by_access_level(user).filter(table=table)
    else:
        return Column.objects.filter(table=table)


def get_table_metadata(user, schema_name, table_name):
    try:
        return Table.objects.filter_by_access_level(user).filter(schema__name=schema_name).get(name=table_name)
    except Table.DoesNotExist:
        return None




import pprint

from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def handle(self, *args, **options):
        pprint.pprint(settings.DATABASES, indent=4)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-22 08:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_files', '0002_depth'),
    ]

    operations = [
        migrations.AlterField(
            model_name='directory',
            name='access_level',
            field=models.CharField(choices=[('PRIVATE', 'Private - access must be granted by group'), ('INTERNAL', 'Internal - logged in users can access'), ('PUBLIC', 'Public - anonymous visitors can access')], max_length=8, verbose_name='Access level'),
        ),
    ]



import subprocess

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('queue', nargs='?', default='default', help='Name of the worker [default=default].')

    def handle(self, *args, **options):
        key = options['queue']
        hostname = f'{settings.DAIQUIRI_APP}_{key}@%h'

        try:
            queue = next(filter(lambda q: q['key'] == key, settings.QUEUES))
        except StopIteration:
            try:
                queue = next(filter(lambda q: q['key'] == key, settings.QUERY_QUEUES))
            except StopIteration as e:
                raise CommandError(f'Queue "{key}" does not exist') from e

        args = [
            'celery',
            '-A', 'config',
            'worker',
            '-Q', key,
            '-c', str(queue.get('concurrency', 1)),
            '-n', hostname,
            '-l', 'info'
        ]

        subprocess.call(args)



import logging

from django.conf import settings
from django.db import OperationalError, ProgrammingError

from .base import BaseDatabaseAdapter

logger = logging.getLogger(__name__)


class PostgreSQLAdapter(BaseDatabaseAdapter):

    DATATYPES = {
        'character': {
            'datatype': 'char',
            'arraysize': True
        },
        'varchar': {
            'datatype': 'char',
            'arraysize': True
        },
        'text': {
            'datatype': 'char',
            'arraysize': True
        },
        'boolean': {
            'datatype': 'boolean',
            'arraysize': False
        },
        'smallint': {
            'datatype': 'short',
            'arraysize': False
        },
        'integer': {
            'datatype': 'int',
            'arraysize': False
        },
        'bigint': {
            'datatype': 'long',
            'arraysize': False
        },
        'real': {
            'datatype': 'float',
            'arraysize': False
        },
        'double precision': {
            'datatype': 'double',
            'arraysize': False
        },
        'spoint':{
            'datatype': 'char',
            'arraysize': True
        },
        '_int2': {
            'datatype': 'short',
            'arraysize': '*'
        },
        '_int4': {
            'datatype': 'int',
            'arraysize': '*'
        },
        '_int8': {
            'datatype': 'long',
            'arraysize': '*'
        },
        '_float4': {
            'datatype': 'float',
            'arraysize': '*'
        },
        '_float8': {
            'datatype': 'double',
            'arraysize': '*'
        },
        '_bool': {
            'datatype': 'boolean',
            'arraysize': '*'
        }
    }

    COLUMNTYPES = {
        'char': 'text',
        'unicodeChar': 'text',
        'boolean': 'boolean',
        'bit': 'boolean',
        # 'unsignedByte': ???, not supported by Postgres... could be solved with pguint extension
        'short': 'smallint',
        'int': 'integer',
        'long': 'bigint',
        'float': 'real',
        'double': 'double precision',
        #'floatComplex': ???, not supported by Postgres
        #'doubleComplex': ???, not supported by Postgres
        'spoint': 'spoint',
    }

    search_stmt_template = '%s::text LIKE %%s'
    search_arg_template = '%%%s%%'

    def fetch_pid(self):
        return self.connection().connection.info.backend_pid

    def escape_identifier(self, identifier):
        # escape quotes within the identifier and quote the string
        return '"{}"'.format(identifier.replace('"', '""'))

    def escape_string(self, string):
        return f"'{string}'"

    def build_query(self, schema_name, table_name, query, timeout, max_records):
        # construct the actual query
        params = {
            'schema': self.escape_identifier(schema_name),
            'table': self.escape_identifier(table_name),
            'query': query,
            'timeout': int(timeout * 1000),
            'max_records': max_records,
            'tablespace': settings.USER_TABLESPACE,
        }

        query = 'SET SESSION statement_timeout TO %(timeout)s; COMMIT; CREATE TABLE %(schema)s.%(table)s'

        if params['tablespace'] is not None:
            query = query + ' TABLESPACE %(tablespace)s AS %(query)s'
        else:
            query = query + ' AS %(query)s'

        if params['max_records'] is not None:
            query = query + ' LIMIT %(max_records)s'

        query = query + ';'

        return query % params


    def build_sync_query(self, query, timeout, max_records):
        # construct the actual query
        params = {
            'query': query,
            'timeout': int(timeout * 1000),
            'max_records': max_records
        }

        if max_records is not None:
            return 'SET SESSION statement_timeout TO {timeout}; COMMIT; {query} LIMIT {max_records};'.format(**params)
        else:
            return 'SET SESSION statement_timeout TO {timeout}; COMMIT; {query};'.format(**params)

    def abort_query(self, pid):
        sql = 'select pg_cancel_backend(%(pid)i)' % {'pid': pid}
        self.execute(sql)

    def fetch_size(self, schema_name, table_name):
        # fetch the size of the table using pg_total_relation_size
        sql = f'SELECT pg_total_relation_size(\'{self.escape_identifier(schema_name)}.{self.escape_identifier(table_name)}\')'  # noqa: E501
        size = self.fetchone(sql)[0]

        # log values and return
        logger.debug('size = %d', size)
        return size

    def fetch_nrows(self, schema_name, table_name):
        # fetch the size of the table using pg_total_relation_size
        sql = f'SELECT reltuples::BIGINT FROM pg_class WHERE oid = \'{self.escape_identifier(schema_name)}.{self.escape_identifier(table_name)}\'::regclass;'  # noqa: E501
        nrows = self.fetchone(sql)[0]

        # log values and return
        logger.debug('nrows = %d', nrows)
        return nrows

    def fetch_tables(self, schema_name):
        # escape input
        escaped_schema_name = self.escape_string(schema_name)

        # prepare sql string
        sql = f'SELECT table_name, table_type FROM information_schema.tables where table_schema = {escaped_schema_name}'

        # log sql string
        logger.debug('sql = "%s"', sql)

        # execute query
        try:
            rows = self.fetchall(sql)
        except OperationalError as e:
            logger.error('Could not fetch from %s (%s)', schema_name, e)
            return []
        else:
            return [{
                'name': row[0],
                'type': 'view' if row[1] == 'VIEW' else 'table'
            } for row in rows]

    def fetch_table(self, schema_name, table_name):
        # prepare sql string
        sql = f'SELECT table_name, table_type FROM information_schema.tables where table_schema = {self.escape_string(schema_name)} AND table_name = {self.escape_string(table_name)}'  # noqa: E501

        # log sql string
        logger.debug('sql = "%s"', sql)

        # execute query
        try:
            row = self.fetchone(sql)
        except OperationalError as e:
            logger.error('Could not fetch %s.%s (%s)', schema_name, table_name, e)
            return {}
        else:
            if row is None:
                logger.info('Could not fetch %s.%s. Check if table and schema exist.', schema_name, table_name)
                return {}
            else:
                return {
                    'name': row[0],
                    'type': 'view' if row[1] == 'VIEW' else 'table'
                }

    def fetch_columns(self, schema_name, table_name):
        logger.debug('fetch_columns %s %s', schema_name, table_name)

        # prepare sql string
        sql = f'SELECT column_name, data_type, udt_name, character_maximum_length, ordinal_position FROM information_schema.columns WHERE table_schema = {self.escape_string(schema_name)} AND table_name = {self.escape_string(table_name)}'  # noqa: E501

        # log sql string
        logger.debug('sql = "%s"', sql)

        # execute query
        try:
            rows = self.fetchall(sql)
        except ProgrammingError as e:
            logger.error('Could not fetch columns from %s.%s (%s)', schema_name, table_name, e)
            return []
        else:
            columns = []
            for row in rows:
                columns.append(self._parse_column(row))

            # check if indexed
            sql = f'SELECT indexdef FROM pg_indexes WHERE schemaname = {self.escape_string(schema_name)} AND tablename = {self.escape_string(table_name)}'  # noqa: E501

            # log sql string
            logger.debug('sql = "%s"', sql)

            try:
                rows = self.fetchall(sql)
            except OperationalError as e:
                logger.error('Could not fetch indexes of %s.%s (%s)', schema_name, table_name, e)
                return columns
            else:
                for column in columns:
                    columnname = '({})'.format(column['name'])
                    if str(rows).find(columnname) > -1:
                        column['indexed'] = True
            return columns

    def fetch_column(self, schema_name, table_name, column_name):
        # prepare sql string
        sql = f'SELECT column_name, data_type, udt_name, character_maximum_length, ordinal_position FROM information_schema.columns WHERE table_schema = {self.escape_string(schema_name)} AND table_name = {self.escape_string(table_name)} AND column_name = {self.escape_string(column_name)}'  # noqa: E501

        # log sql string
        logger.debug('sql = "%s"', sql)

        # execute query
        try:
            row = self.fetchone(sql)
        except ProgrammingError as e:
            logger.error('Could not fetch %s.%s.%s (%s)', schema_name, table_name, column_name, e)
            return {}
        else:
            if row is None:
                logger.info('Could not fetch %s.%s.%s. Check if the schema exists.', schema_name, table_name, column_name)  # noqa: E501
                return {}
            else:
                column = self._parse_column(row)

            # check if indexed
            sql = f'SELECT indexdef FROM pg_indexes WHERE schemaname = {self.escape_string(schema_name)} AND tablename = {self.escape_string(table_name)}'  # noqa: E501

            # log sql string
            logger.debug('sql = "%s"', sql)

            try:
                rows = self.fetchall(sql)
            except OperationalError as e:
                logger.error('Could not fetch indexes of %s.%s.%s (%s)', schema_name, table_name, column_name, e)
                return column
            else:
                columnname = '({})'.format(column['name'])
                if str(rows).find(columnname) > -1:
                    column['indexed'] = True

            return column

    def fetch_column_names(self, schema_name, table_name):
        logger.debug('schema_name = "%s"', schema_name)
        logger.debug('table_name = "%s"', table_name)

        # prepare sql string
        sql = f'SELECT column_name FROM information_schema.columns where table_schema = {self.escape_string(schema_name)} AND table_name = {self.escape_string(table_name)}'  # noqa: E501

        # log values and return
        logger.debug('sql = "%s"', sql)
        return [column[0] for column in self.fetchall(sql)]

    def rename_table(self, schema_name, table_name, new_table_name):
        sql = f'ALTER TABLE {self.escape_identifier(schema_name)}.{self.escape_identifier(table_name)} RENAME TO {self.escape_identifier(new_table_name)};'  # noqa: E501

        # log sql string and execute
        logger.debug('sql = "%s"', sql)
        self.execute(sql)

    def _parse_column(self, row):
        column_name, data_type, udt_name, character_maximum_length, ordinal_position = row

        column = {
            'name': column_name
        }

        if udt_name.lower() in self.DATATYPES: # universal types: int2, int4, int8, _float8, ...
            datatype = self.DATATYPES[udt_name.lower()]

        elif data_type.lower() in self.DATATYPES: # postgres types: smallint, bigint, ARRAY, ...
            datatype = self.DATATYPES[data_type.lower()]

        else:
            datatype = None

        if datatype:
            if '_' in udt_name.lower(): # ARRAY
                column['datatype'] = datatype['datatype'] # the _ is still needed, will be removed by the serializer.
                column['arraysize'] = '*'

            else:
                column['datatype'] = datatype['datatype']

                if datatype['arraysize'] is True:
                    column['arraysize'] = character_maximum_length
                else:
                    column['arraysize'] = None

        else:
            column['datatype'] = 'char'
            column['arraysize'] = 32

        column['order'] = ordinal_position

        # log values and return
        logger.debug('row = %s', row)
        logger.debug('column = %s', column)
        return column



from django.conf import settings

from daiquiri.core.utils import import_class


def DatabaseAdapter():
    return import_class(settings.ADAPTER_DATABASE)('data', settings.DATABASES['data'])


def DownloadAdapter():
    return import_class(settings.ADAPTER_DOWNLOAD)('data', settings.DATABASES['data'])



import logging

from .base import BaseDownloadAdapter

logger = logging.getLogger(__name__)


class MysqldumpAdapter(BaseDownloadAdapter):

    def set_args(self, schema_name, table_name, data_only=False):
        self.args = ['mysqldump']
        if data_only:
            self.args += ['--compact', '--skip-extended-insert']

        if self.database_config.get('USER'):
            self.args.append('--user={USER}'.format(**self.database_config))

        if self.database_config.get('PASSWORD'):
            self.args.append('--password={PASSWORD}'.format(**self.database_config))

        if self.database_config.get('HOST'):
            self.args.append('--host={HOST}'.format(**self.database_config))

        if self.database_config.get('PORT'):
            self.args.append('--port={PORT}'.format(**self.database_config))

        self.args.append(schema_name)
        self.args.append(table_name)



import logging

from .base import BaseDownloadAdapter

logger = logging.getLogger(__name__)


class PgDumpAdapter(BaseDownloadAdapter):

    def set_args(self, schema_name, table_name, data_only=False):
        # command line for pg_dump:
        # pg_dump ... --dbname=postgresql://user:password@host:port/database --table=schema.table
        # pg_dump ... --user=user --host=host --port=port --dbname=database --table=schema.table

        self.args = ['pg_dump', '--no-owner']
        if data_only:
            self.args += ['--data-only', '--inserts']

        if self.database_config.get('PASSWORD'):
            if self.database_config.get('PORT'):
                dbname = '--dbname=postgresql://%(USER)s:%(PASSWORD)s@%(HOST)s:%(PORT)s/%(NAME)s'
            else:
                dbname = '--dbname=postgresql://%(USER)s:%(PASSWORD)s@%(HOST)s/%(NAME)s'

            self.args.append(dbname % self.database_config)
        else:
            if self.database_config.get('USER'):
                self.args.append('--user={USER}'.format(**self.database_config))

            if self.database_config.get('HOST'):
                self.args.append('--host={HOST}'.format(**self.database_config))

            if self.database_config.get('PORT'):
                self.args.append('--port=%(PORT)d' % self.database_config)

            self.args.append('--dbname={NAME}'.format(**self.database_config))

        self.args.append(f'--table="{schema_name}"."{table_name}"')



import pytest

from django.test import override_settings
from django.urls import reverse
from django.utils.http import urlencode

users = (
    ('admin', 'admin'),
    ('user', 'user'),
    ('test', 'test'),
    ('evil', 'evil'),
    ('anonymous', None),
)

url_names = {
    'list': 'tap:sync-list'
}

instances = [
    '679b5ce8-bbcf-41c2-ba45-2340910bcfae',
    '5527f2b0-7130-466d-b8d2-dbf6c5f700bf',
    'c87926e4-8e5f-4032-a764-826b6a39c171',
    '1b6c93ef-4161-4402-b1a7-d1237657e807'
]

queries_public = [
    'SELECT ra, dec, parallax, id FROM daiquiri_data_obs.stars'
]

queries_internal = [
    'SELECT x, y, z, vx, vy, vz, id FROM daiquiri_data_sim.halos'
]

queries_private = [
    'SELECT id, "bool", "array", matrix FROM daiquiri_data_test.test'
]

uws_ns = '{http://www.ivoa.net/xml/UWS/v1.0}'


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_public)
def test_create_post_public(db, client, username, password, query):
    '''
    GET /{jobs} with an ulrencodes set of KEY=VALUE as query params
    creates a job with these parameters and returns a VOTable.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list']) + '?' + urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query
    })
    response = client.get(url)

    assert response.status_code == 200


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_internal)
def test_create_post_internal(db, client, username, password, query):
    '''
    GET /{jobs} with an ulrencodes set of KEY=VALUE as query params
    creates a job with these parameters and returns a VOTable.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list']) + '?' + urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query
    })
    response = client.get(url)

    if username != 'anonymous':
        assert response.status_code == 200
    else:
        assert response.status_code == 400


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_private)
def test_create_post_private(db, client, username, password, query):
    '''
    GET /{jobs} with an ulrencodes set of KEY=VALUE as query params
    creates a job with these parameters and returns a VOTable.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list']) + '?' + urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query
    })
    response = client.get(url)

    if username == 'test':
        assert response.status_code == 200
    else:
        assert response.status_code == 400


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_public)
def test_post_job_list_create_public(db, client, username, password, query):
    '''
    POST /{jobs} with an application/x-www-form-urlencoded set of KEY=VALUE
    creates a job with these parameters and returns a VOTable.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list'])
    response = client.post(url, urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query
    }), content_type='application/x-www-form-urlencoded')

    assert response.status_code == 200


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_internal)
def test_post_job_list_create_internal(db, client, username, password, query):
    '''
    POST /{jobs} with an application/x-www-form-urlencoded set of KEY=VALUE
    creates a job with these parameters and returns a VOTable.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list'])
    response = client.post(url, urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query
    }), content_type='application/x-www-form-urlencoded')

    if username != 'anonymous':
        assert response.status_code == 200
    else:
        assert response.status_code == 400


@override_settings(QUERY_ANONYMOUS=True)
@pytest.mark.parametrize(('username', 'password'), users)
@pytest.mark.parametrize('query', queries_private)
def test_post_job_list_create_private(db, client, username, password, query):
    '''
    POST /{jobs} with an application/x-www-form-urlencoded set of KEY=VALUE
    creates a job with these parameters and returns a VOTable.
    '''
    client.login(username=username, password=password)

    url = reverse(url_names['list'])
    response = client.post(url, urlencode({
        'LANG': 'adql-2.0',
        'QUERY': query
    }), content_type='application/x-www-form-urlencoded')

    if username == 'test':
        assert response.status_code == 200
    else:
        assert response.status_code == 400



import logging
import warnings

from django.db import connections

logger = logging.getLogger(__name__)


class BaseDatabaseAdapter:

    def __init__(self, database_key, database_config):
        self.database_key = database_key
        self.database_config = database_config

    def connection(self):
        return connections[self.database_key]

    def execute(self, sql):
        return self.connection().cursor().execute(sql)

    def fetchone(self, sql, args=None, as_dict=False):
        cursor = self.connection().cursor()

        if args:
            cursor.execute(sql, args)
        else:
            cursor.execute(sql)

        if as_dict:
            columns = cursor.description
            row = cursor.fetchone()
            if row:
                return {column.name: value for column, value in zip(columns, row)}
            else:
                return None
        else:
            return cursor.fetchone()

    def fetchall(self, sql, args=None, as_dict=False):
        cursor = self.connection().cursor()

        if args:
            cursor.execute(sql, args)
        else:
            cursor.execute(sql)

        if as_dict:
            columns = cursor.description
            return [
                {column.name: value for column, value in zip(columns, row)}
                for row in cursor.fetchall()
            ]
        else:
            return cursor.fetchall()

    def fetch_pid(self):
        raise NotImplementedError()

    def escape_identifier(self, identifier):
        raise NotImplementedError()

    def escape_string(self, string):
        raise NotImplementedError()

    def build_query(self, schema_name, table_name, query, timeout):
        raise NotImplementedError()

    def build_sync_query(self, query, timeout, max_records):
        raise NotImplementedError()

    def submit_query(self, sql):
        self.execute(sql)

    def abort_query(self, pid):
        raise NotImplementedError()

    def count_rows(self, schema_name, table_name, column_names=None, search=None, filters=None):
        # if no column names are provided get all column_names from the table
        if not column_names:
            column_names= self.fetch_column_names(schema_name, table_name)

        # create a list of escaped columns
        escaped_column_names = [self.escape_identifier(column_name) for column_name in column_names]

        # prepare sql string
        sql = f'SELECT COUNT(*) FROM {self.escape_identifier(schema_name)}.{self.escape_identifier(table_name)}'
        sql_args = []

        # process filtering
        sql, sql_args = self._process_filtering(sql, sql_args, search, filters, escaped_column_names)

        return self.fetchone(sql, args=sql_args)[0]

    def fetch_rows(self, schema_name, table_name, column_names=None, ordering=None, page=1, page_size=10,
                   search=None, filters=None):

        # if no column names are provided get all column_names from the table
        if not column_names:
            column_names = self.fetch_column_names(schema_name, table_name)

        # create a list of escaped columns
        escaped_column_names = [self.escape_identifier(column_name) for column_name in column_names]

        # init sql string and sql_args list
        sql = 'SELECT {columns} FROM {schema}.{table}'.format(
            schema=self.escape_identifier(schema_name),
            table=self.escape_identifier(table_name),
            columns=', '.join(escaped_column_names),
        )
        sql_args = []

        # process filtering
        sql, sql_args = self._process_filtering(sql, sql_args, search, filters, escaped_column_names)

        # process ordering
        sql = self._process_ordering(sql, ordering, escaped_column_names)

        # process page and page_size
        if page_size > 0:
            sql += f' LIMIT {page_size} OFFSET {(int(page) - 1) * int(page_size)}'

        return self.fetchall(sql, args=sql_args)

    def fetch_row(self, schema_name, table_name, column_names=None, search=None, filters=None):

        # if no column names are provided get all column_names from the table
        if not column_names:
            column_names = self.fetch_column_names(schema_name, table_name)

        # create a list of escaped columns
        escaped_column_names = [self.escape_identifier(column_name) for column_name in column_names]

        # prepare sql string
        sql = 'SELECT {columns} FROM {schema}.{table}'.format(
            schema=self.escape_identifier(schema_name),
            table=self.escape_identifier(table_name),
            columns=', '.join(escaped_column_names),
        )
        sql_args = []

        # process filtering
        sql, sql_args = self._process_filtering(sql, sql_args, search, filters, escaped_column_names)

        return self.fetchone(sql, args=sql_args)

    def fetch_dict(self, schema_name, table_name, column_names=None, search=None, filters=None):

        # if no column names are provided get all column_names from the table
        if not column_names:
            column_names = self.fetch_column_names(schema_name, table_name)

        row = self.fetch_row(schema_name, table_name, column_names, search, filters)

        if row:
            return dict(zip(column_names, row))
        else:
            return {}

    def fetch_size(self, schema_name, table_name):
        raise NotImplementedError()

    def fetch_nrows(self, schema_name, table_name):
        raise NotImplementedError()

    def fetch_tables(self, schema_name):
        raise NotImplementedError()

    def fetch_table(self, schema_name, table_name):
        raise NotImplementedError()

    def fetch_columns(self, schema_name, table_name):
        raise NotImplementedError()

    def fetch_column(self, schema_name, table_name, column_name):
        raise NotImplementedError()

    def fetch_column_names(self, schema_name, table_name):
        raise NotImplementedError()

    def create_user_schema_if_not_exists(self, schema_name):
        # escape input
        escaped_schema_name = self.escape_identifier(schema_name)

        # prepare sql string
        sql = f'CREATE SCHEMA IF NOT EXISTS {escaped_schema_name}'

        # log sql string
        logger.debug('sql = "%s"', sql)

        with warnings.catch_warnings():
            warnings.simplefilter('ignore')
            self.execute(sql)

    def create_table(self, schema_name, table_name, columns):

        # check column types
        for column in columns:
            if self.COLUMNTYPES.get(column['datatype']) is None:
                raise TypeError("Column {name} is of type {datatype}. {datatype} is not ".format(**column) +
                                "supported by Postgres, the table can not be created.")

        # prepare sql string
        sql = 'CREATE TABLE {schema}.{table} ({columns});'.format(
            schema=self.escape_identifier(schema_name),
            table=self.escape_identifier(table_name),
            columns=', '.join([
                '{column_name} {column_type}'.format(
                    column_name=self.escape_identifier(column['name']),
                    column_type=self.COLUMNTYPES[column['datatype']],
                ) for column in columns]),
        )

        # log sql string and execute
        logger.debug('sql = "%s"', sql)
        self.execute(sql)

    def rename_table(self, schema_name, table_name, new_table_name):
        raise NotImplementedError()

    def drop_table(self, schema_name, table_name):
        sql = f'DROP TABLE IF EXISTS {self.escape_identifier(schema_name)}.{self.escape_identifier(table_name)};'

        # log sql string and execute
        logger.debug('sql = "%s"', sql)
        self.execute(sql)

    def insert_rows(self, schema_name, table_name, columns, rows, mask=None):
        # create a list of escaped columns
        escaped_column_names = [self.escape_identifier(column['name']) for column in columns]

        # create a list of escaped value tuples
        escaped_rows = []
        if mask is None:
            for row in rows:
                # all values are escaped with quotes
                escaped_cells = []
                for cell in row:
                    escaped_cells.append(self._escape_cell(cell))

                escaped_row = ', '.join(escaped_cells)
                escaped_rows.append(f'({escaped_row})')

        else:
            for row, mask_row in zip(rows, mask):
                # all values are escaped with quotes
                escaped_cells = []
                for cell, mask_cell in zip(row, mask_row):
                    escaped_cells.append(self._escape_cell(cell, mask_cell))

                escaped_row = ', '.join(escaped_cells)
                escaped_rows.append(f'({escaped_row})')

        # prepare sql string
        sql = 'INSERT INTO {schema}.{table} ({columns}) VALUES {values}'.format(
            schema=self.escape_identifier(schema_name),
            table=self.escape_identifier(table_name),
            columns=', '.join(escaped_column_names),
            values=', '.join(escaped_rows),
        )

        # log sql string and execute
        logger.debug('sql = "%s"', sql)
        self.execute(sql)

    def _process_filtering(self, sql, sql_args, search, filters, escaped_column_names):
        # prepare lists for the WHERE statements
        where_stmts = []
        where_args = []

        if search:
            # append a OR condition for every column
            search_stmts = []
            search_args = []
            for escaped_column_name in escaped_column_names:
                # search_stmt_template and search_arg_template are set differently for mysql and postgres
                search_stmts.append(self.search_stmt_template % escaped_column_name)
                search_args.append(self.search_arg_template % search)

            if search_stmts:
                where_stmts.append('(' + ' OR '.join(search_stmts) + ')')
                where_args += search_args

        if filters:
            for column_name, column_filter in filters.items():
                # escape the column_name for this column
                escaped_column_name = self.escape_identifier(column_name)

                # check if the filter is a list or a string
                if isinstance(column_filter, str):
                    filter_list = [column_filter]
                elif isinstance(column_filter, list):
                    filter_list = column_filter
                else:
                    raise RuntimeError(f'Unsupported filter for column "{column_name}"')

                # append a OR condition for every entry in the list
                filter_stmts = []
                filter_args = []
                for filter_string in filter_list:
                    filter_stmts.append(escaped_column_name + ' = %s')
                    filter_args.append(filter_string)

                if filter_stmts:
                    where_stmts.append('(' + ' OR '.join(filter_stmts) + ')')
                    where_args += filter_args

        # connect the where statements with AND and append to the sql string
        if where_stmts:
            sql += ' WHERE ' + ' AND '.join(where_stmts)
            sql_args += where_args

        return sql, sql_args

    def _process_ordering(self, sql, ordering, escaped_column_names):
        if ordering:
            if ordering.startswith('-'):
                escaped_ordering_column, ordering_direction = self.escape_identifier(ordering[1:]), 'DESC'
            else:
                escaped_ordering_column, ordering_direction = self.escape_identifier(ordering), 'ASC'

            if escaped_ordering_column in escaped_column_names:
                sql += f' ORDER BY {escaped_ordering_column} {ordering_direction}'

        return sql

    def _escape_cell(self, cell, mask_cell=None):
        if mask_cell:
            return 'NULL'
        else:
            if isinstance(cell, str):
                # this is the case for fields datatype="char" and arraysize="*"
                value = cell
            elif cell.dtype.char == 'S':
                # chars need to be decoded
                value = cell.decode()
            elif cell.dtype.char == '?':
                # booleans need to be converted to 0 or 1, or quoting will fail
                value = 1 if cell else 0
            else:
                value = cell

            return self.escape_string(value)



from django.apps import AppConfig


class AuthConfig(AppConfig):
    name = 'daiquiri.auth'
    label = 'daiquiri_auth'
    verbose_name = 'User Profiles'

    def ready(self):
        from . import handlers  # noqa: F401



from rest_framework import serializers

from daiquiri.stats.models import Record


class RecordSerializer(serializers.ModelSerializer):


    class Meta:
        model = Record
        fields = '__all__'



from django.utils.translation import gettext_lazy as _

ACCESS_LEVEL_PRIVATE = 'PRIVATE'
ACCESS_LEVEL_INTERNAL = 'INTERNAL'
ACCESS_LEVEL_PUBLIC = 'PUBLIC'

ACCESS_LEVEL_CHOICES = (
    (ACCESS_LEVEL_PRIVATE, _('Private - access must be granted by group')),
    (ACCESS_LEVEL_INTERNAL, _('Internal - logged in users can access')),
    (ACCESS_LEVEL_PUBLIC, _('Public - anonymous visitors can access'))
)

GROUPS = {
    'contact_manager': [
        'daiquiri_contact.view_contactmessage',
        'daiquiri_contact.add_contactmessage',
        'daiquiri_contact.change_contactmessage',
        'daiquiri_contact.delete_contactmessage'
    ],
    'metadata_manager': [
        'daiquiri_metadata.view_schema',
        'daiquiri_metadata.add_schema',
        'daiquiri_metadata.change_schema',
        'daiquiri_metadata.delete_schema',
        'daiquiri_metadata.view_table',
        'daiquiri_metadata.add_table',
        'daiquiri_metadata.change_table',
        'daiquiri_metadata.delete_table',
        'daiquiri_metadata.view_column',
        'daiquiri_metadata.add_column',
        'daiquiri_metadata.change_column',
        'daiquiri_metadata.delete_column',
        'daiquiri_metadata.view_function',
        'daiquiri_metadata.add_function',
        'daiquiri_metadata.change_function',
        'daiquiri_metadata.delete_function',
        'auth.view_group'
    ],
    'stats_manager': [
        'daiquiri_stats.view_record'
    ],
    'query_manager': [
        'daiquiri_query.view_example',
        'daiquiri_query.add_example',
        'daiquiri_query.change_example',
        'daiquiri_query.delete_example',
        'auth.view_group'
    ],
    'user_manager': [
        'daiquiri_auth.view_profile',
        'daiquiri_auth.change_profile',
        'auth.view_group'
    ]
}



# Generated by Django 3.2.12 on 2022-02-04 14:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('daiquiri_stats', '0006_alter_record_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='record',
            name='resource',
            field=models.JSONField(),
        ),
    ]



from daiquiri.core.renderers import XMLRenderer
from daiquiri.core.renderers.datacite import DataciteRendererMixin
from daiquiri.core.renderers.dublincore import DublincoreRendererMixin
from daiquiri.core.renderers.voresource import VoresourceRendererMixin


class OaiRenderer(DublincoreRendererMixin, DataciteRendererMixin, VoresourceRendererMixin, XMLRenderer):

    def render_document(self, data, accepted_media_type=None, renderer_context=None):
        self.start('oai:OAI-PMH', {
            'xmlns:oai': 'http://www.openarchives.org/OAI/2.0/',
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xsi:schemaLocation': 'http://www.openarchives.org/OAI/2.0/ http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd'
        })
        self.node('oai:responseDate', {}, data['responseDate'])

        request_arguments = data['arguments']
        for error_code, _ in data['errors']:
            if error_code in ['badVerb', 'badArgument']:
                request_arguments = {}

        self.node('oai:request', request_arguments, data['baseUrl'])

        if data['errors']:
            self.render_errors(data['errors'])
        elif data['verb'] == 'GetRecord':
            self.render_get_record(data['response'])
        elif data['verb'] == 'Identify':
            self.render_identify(data['response'], data['baseUrl'])
        elif data['verb'] == 'ListIdentifiers':
            self.render_list_identifiers(data['response']['items'], data['response']['resumptionToken'])
        elif data['verb'] == 'ListMetadataFormats':
            self.render_list_metadata_formats(data['response'])
        elif data['verb'] == 'ListRecords':
            self.render_list_records(data['response']['items'], data['response']['resumptionToken'])
        elif data['verb'] == 'ListSets':
            self.render_list_sets(data['response'])

        self.end('oai:OAI-PMH')

    def render_errors(self, errors):
        for error_code, error_message in errors:
            self.node('error', {'code': error_code}, error_message)

    def render_get_record(self, item):
        self.start('oai:GetRecord')
        self.render_record(item)
        self.end('oai:GetRecord')

    def render_identify(self, repository_metadata, base_url):
        self.start('oai:Identify')
        self.node('oai:repositoryName', {}, repository_metadata.get('repository_name'))
        self.node('oai:baseURL', {}, base_url)
        self.node('oai:protocolVersion', {}, '2.0')
        self.node('oai:adminEmail', {}, repository_metadata['admin_email'])
        self.node('oai:earliestDatestamp', {}, repository_metadata.get('earliest_datestamp').strftime('%Y-%m-%dT%H:%M:%SZ'))  # noqa: E501
        self.node('oai:deletedRecord', {}, repository_metadata.get('deleted_record'))
        self.node('oai:granularity', {}, 'YYYY-MM-DDThh:mm:ssZ')
        self.render_identify_description(repository_metadata)
        self.end('oai:Identify')

    def render_identify_description(self, repository_metadata):
        self.start('oai:description')
        if repository_metadata['identifier'] is not None:
            self.render_oai_identifier(repository_metadata.get('identifier'))
        self.end('oai:description')

        self.start('oai:description')
        if repository_metadata['registry'] is not None:
            self.render_voresource(repository_metadata.get('registry'))
        self.end('oai:description')

    def render_oai_identifier(self, identifier_metadata):
        self.start('oai-identifier', {
            'xmlns': 'http://www.openarchives.org/OAI/2.0/oai-identifier',
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xsi:schemaLocation': 'http://www.openarchives.org/OAI/2.0/oai-identifier http://www.openarchives.org/OAI/2.0/oai-identifier.xsd'
        })
        self.node('scheme', {}, identifier_metadata.get('scheme'))
        self.node('repositoryIdentifier', {}, identifier_metadata.get('repository_identifier'))
        self.node('delimiter', {}, identifier_metadata.get('delimiter'))
        self.node('sampleIdentifier', {}, identifier_metadata.get('sample_identifier'))
        self.end('oai-identifier')

    def render_list_identifiers(self, items, resumption_token):
        self.start('oai:ListIdentifiers')
        for item in items:
            self.render_header(item['header'])

        if resumption_token:
            self.node('oai:resumptionToken', {
                'oai:expirationDate': resumption_token.get('expirationDate'),
                'oai:completeListSize': resumption_token.get('completeListSize'),
                'oai:cursor': resumption_token.get('cursor')
            }, resumption_token['token'])

        self.end('oai:ListIdentifiers')

    def render_list_metadata_formats(self, metadata_formats):
        self.start('oai:ListMetadataFormats')
        for metadata_format in metadata_formats:
            self.start('oai:metadataFormat')
            self.node('oai:metadataPrefix', {}, metadata_format['prefix'])
            self.node('oai:schema', {}, metadata_format.get('schema'))
            self.node('oai:metadataNamespace', {}, metadata_format.get('namespace'))
            self.end('oai:metadataFormat')
        self.end('oai:ListMetadataFormats')

    def render_list_records(self, items, resumption_token):
        self.start('oai:ListRecords')
        for item in items:
            self.render_record(item)

        if resumption_token:
            self.node('oai:resumptionToken', {
                'oai:expirationDate': resumption_token.get('expirationDate'),
                'oai:completeListSize': resumption_token.get('completeListSize'),
                'oai:cursor': resumption_token.get('cursor')
            }, resumption_token['token'])

        self.end('oai:ListRecords')

    def render_list_sets(self, data):
        self.start('oai:ListSets')
        for oai_set in data['oai_sets']:
            self.start('oai:set')
            self.node('oai:setSpec', {}, oai_set['setSpec'])
            self.node('oai:setName', {}, oai_set['setName'])
            if oai_set['setDescription'] is not None:
                self.node('oai:setDescription', {}, oai_set['setDescription'])
            self.end('oai:set')
        self.end('oai:ListSets')

    def render_record(self, record):
        self.start('oai:record')
        self.render_header(record['header'])
        if record['metadata'] is not None:
            self.start('oai:metadata')
            self.render_metadata(record['metadata'])
            self.end('oai:metadata')
        self.end('oai:record')

    def render_header(self, header):
        self.start('oai:header', {'status': 'deleted'} if header['deleted'] else {})
        self.node('oai:identifier', {}, header['identifier'])
        self.node('oai:datestamp', {}, header['datestamp'])
        for spec in header.get('setSpec', []):
            self.node('oai:setSpec', {}, spec)
        self.end('oai:header')

    def render_metadata(self, metadata):
        raise NotImplementedError()


class DublincoreRenderer(OaiRenderer):

    def render_metadata(self, metadata):
        self.render_dublincore(metadata)


class OaiDataciteRenderer(OaiRenderer):

    def render_metadata(self, metadata):
        self.start('oai_datacite', {
            'xmlns': 'http://schema.datacite.org/oai/oai-1.0/',
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xsi:schemaLocation': 'http://schema.datacite.org/oai/oai-1.0/ oai_datacite.xsd'
        })
        self.start('payload')
        self.render_datacite(metadata)
        self.end('payload')
        self.end('oai_datacite')


class DataciteRenderer(OaiRenderer):

    def render_metadata(self, metadata):
        self.render_datacite(metadata)


class VoresourceRenderer(OaiRenderer):

    def render_metadata(self, metadata):
        self.render_voresource(metadata)



import os

import daiquiri.core.env as env

LOG_LEVEL = env.get('LOG_LEVEL')
LOG_DIR = env.get('LOG_DIR')

if LOG_DIR:
    LOGGING = {
        'version': 1,
        'disable_existing_loggers': True,
        'filters': {
            'require_debug_false': {
                '()': 'django.utils.log.RequireDebugFalse'
            },
            'require_debug_true': {
                '()': 'django.utils.log.RequireDebugTrue'
            }
        },
        'formatters': {
            'default': {
                'format': '[%(asctime)s] %(levelname)s: %(message)s'
            },
            'name': {
                'format': '[%(asctime)s] %(levelname)s %(name)s %(funcName)s: %(message)s'
            },
            'console': {
                'format': '[%(asctime)s] %(message)s'
            }
        },
        'handlers': {
            'mail_admins': {
                'level': 'ERROR',
                'filters': ['require_debug_false'],
                'class': 'django.utils.log.AdminEmailHandler'
            },
            'error_log': {
                'level': 'ERROR',
                'class': 'logging.FileHandler',
                'filename': os.path.join(LOG_DIR, 'error.log'),
                'formatter': 'default'
            },
            'daiquiri_log': {
                'level': 'DEBUG',
                'class': 'logging.FileHandler',
                'filename': os.path.join(LOG_DIR, 'daiquiri.log'),
                'formatter': 'name'
            },
            'query_log': {
                'level': 'DEBUG',
                'class': 'logging.FileHandler',
                'filename': os.path.join(LOG_DIR, 'query.log'),
                'formatter': 'default'
            },
            'sql_log': {
                'level': 'DEBUG',
                'filters': ['require_debug_true'],
                'class': 'logging.FileHandler',
                'filename': os.path.join(LOG_DIR, 'sql.log'),
                'formatter': 'default'
            },
            'rules_log': {
                'level': 'DEBUG',
                'filters': ['require_debug_true'],
                'class': 'logging.FileHandler',
                'filename': os.path.join(LOG_DIR, 'rules.log'),
                'formatter': 'default'
            },
            'console': {
                'level': 'DEBUG',
                'filters': ['require_debug_true'],
                'class': 'logging.StreamHandler',
                'formatter': 'console'
            }
        },
        'loggers': {
            'django': {
                'handlers': ['console'],
                'level': 'INFO',
                'propagate': False
            },
            'django.request': {
                'handlers': ['mail_admins', 'error_log'],
                'level': 'ERROR',
                'propagate': True
            },
            'django.db.backends': {
                'handlers': ['sql_log'],
                'level': LOG_LEVEL,
                'propagate': False
            },
            'daiquiri': {
                'handlers': ['daiquiri_log'],
                'level': LOG_LEVEL,
                'propagate': False
            },
            'query': {
                'handlers': ['query_log'],
                'level': LOG_LEVEL,
                'propagate': False
            },
            'rules': {
                'handlers': ['rules_log'],
                'level': LOG_LEVEL,
                'propagate': False
            }
        }
    }



from rest_framework.routers import DynamicRoute, Route, SimpleRouter


class JobRouter(SimpleRouter):
    '''
    A dedicated router for UWS services. The main difference is that a POST on an instance
    maps to update, not PUT. Also actions are removed.
    '''

    routes = [
        Route(
            url=r'^{prefix}$',
            mapping={
                'get': 'list',
                'post': 'create'
            },
            name='{basename}-list',
            detail=False,
            initkwargs={'suffix': 'List'}
        ),
        Route(
            url=r'^{prefix}/{lookup}$',
            mapping={
                'get': 'retrieve',
                'post': 'update',
                'delete': 'destroy'
            },
            name='{basename}-detail',
            detail=True,
            initkwargs={'suffix': 'Instance'}
        ),
        DynamicRoute(
            url=r'^{prefix}/{lookup}/{url_path}$',
            name='{basename}-{url_name}',
            detail=True,
            initkwargs={}
        )
    ]
